// FILE:  interchange.h

#include <stream.h>
#include <string.h>
#include <strings.h>
#include "/usr/include/xedip.h"


// 
// CLASS: Interchange
//
// This class provides access to an EDI interchange.
//


typedef enum { X12, EDIFACT, OTHER } edi_type;
 
typedef  char   *string;
typedef  string octet;
typedef  string teletex;
typedef  string printable;
typedef  string utc_time;
typedef  string boolean;

#define	ELEMENT_SEPARATOR_POS	        3
#define	DATA_ELE_SEP_POS	        3
#define	CONTENT_SIZE                    200
#define	INTER_TYPE_SIZE                 4
#define	MAX_FIELD_SIZE                  40
#define	BODYPART_TYPE_SIZE              80
#define EDI_MESSAGE_TYPE_SIZE           40
#define TOKEN_SIZE                      36 

// The following are max sizes for x.435 fields being filled.
#define AUTH_INFO_QUAL_SIZE		3
#define AUTH_INFO_SIZE			11
#define X12_ID_QUAL_SIZE		3
#define SENDER_ID_CODE_SIZE		16
#define RECIP_ID_CODE_SIZE		16
#define DATE_PREP_SIZE	 		7
#define TIME_PREP_SIZE			5
#define SYNTAX_ID_STRING_SIZE		2

#define INTERCHANGE_ID_SIZE    	        36 
#define ID_QUALIFIER_SIZE      	        5    
#define ROUTING_ADDR_SIZE      	        15   
#define ELEMENT_SEPARATOR_SIZE          2
#define SEGMENT_TERMINATOR_SIZE         2
#define DECIMAL_NOTATION_SIZE           2
#define RELEASE_INDICATOR_SIZE          2
#define DECIMAL_NOTATION_SIZE           2
#define RESERVED_SIZE                   2
#define SYNTAX_ID_SIZE                  5
#define SYNTAX_VERSION_SIZE             6
#define TIME_SIZE                       7
#define DATE_SIZE                       6
#define INTERCHANGE_CNTL_SIZE           15
#define RECIP_REF_SIZE                  15
#define RECIP_REF_QUALIFIER_SIZE        15
#define APPLICATION_REF_SIZE            15
#define PROCESS_PRIORITY_CODE_SIZE      2
#define ACK_REQUEST_SIZE                2 
#define COMM_AGREE_SIZE                 36
#define TEST_INDICATOR_SIZE             2

// Class Definition for INTERCHANGE.

class Interchange {
	char *contents;
	char *recipient;
        char *bodypart_type;  // edifact-ISO646 or ansiX12-ISO646
                              // only two supported
        char *edi_message_type;  // EDIFACT, ANSIX12, UNTDI
                                 // First two only supported. 
        // Recipients
         // Interchange Sender
         teletex  recipient_ID_code;
         teletex  recipient_ID_qualifier;
         teletex  recipient_routing_addr;
         
         // Recipient Reference
         teletex recipient_reference;
         teletex recipient_reference_qualifier;

         // Interchange Control Reference
         teletex interchange_control_reference;

         // Processing Priority Code
         teletex process_priority_code;

         // Acknowledgement Request
         boolean acknowledgement_request;
 
         // Communications Agreement ID
         teletex comm_agreement_ID;
         // Test Indicator
         boolean test_indicator;

         // Authorization Information
         teletex authorization_info;
         teletex authorization_info_qualifier;

        // Service String Advice subelements
        octet  comp_data_element_separator;
        octet  data_element_separator;
        octet  decimal_notation;
        octet  release_indicator;
        octet  reserved;
        octet  segment_terminator;

        // Syntax Identifier
        teletex    syntax_ID_string;
        printable  syntax_version;

        // Interchange Sender
        teletex  sender_ID_code;
        teletex  sender_ID_qualifier;
        teletex  sender_routing_addr;

        // Date and Time of Preparation
        utc_time  date_preparation;
        utc_time  time_preparation;

        // Application Reference;
        teletex  application_ref;

public:

                 ~Interchange(); // destructor to free space.
		  Interchange( char * ); // address of interchange
	char	  *content();
        char      temp_contents[CONTENT_SIZE];

       // Determine what type the interchange is (i.e., X12 or EDIFACT)
       // and fill the corresponding X.435 fields.
	STATUS	parse_interchange();
	void	parse_edi_names();

        teletex   get_recipient_ID_code();
        teletex   get_recipient_ID_qualifier();
        teletex   get_recipient_routing_addr();
        teletex   get_recipient_reference();
        teletex   get_recipient_reference_qualifier();
        teletex   get_interchange_control_reference();
        teletex   get_process_priority_code();
        boolean   get_acknowledgement_request();
        teletex   get_comm_agreement_ID();
        boolean   get_test_indicator();
        teletex   get_authorization_info();
        teletex   get_authorization_info_qualifier();

        octet     get_comp_data_element_separator();
        octet     get_data_element_separator();
        octet     get_decimal_notation();   
        octet     get_release_indicator();  
        octet     get_reserved();           
        octet     get_segment_terminator(); 
        teletex   get_syntax_id_string();
        printable get_syntax_version();
        teletex   get_sender_ID_code();
        teletex   get_sender_ID_qualifier();
        teletex   get_sender_routing_addr();
        utc_time  get_date_preparation();
        utc_time  get_time_preparation();
        teletex   get_application_ref();

	// non interchange (sub) elements.
	char *get_bodypart_type();
        char *get_edi_message_type();
        char *get_interchange_type(char *);

        // Logging method prototype.
        void log_interchange( LOGLEVEL, ofstream * );

#ifdef	DEBUG
	void	display_interchange();
#endif	

private:
	void parse_X12_interchange();
        void parse_EDIFACT_interchange();

};

//
// Interchange()
//
// Class Members:  Interchange
// Parameters:	   cnt
// Called By:	   public/constructor
//
// This method initializes the class member identifying the interchange.
//

inline Interchange::Interchange( char *cnt ) { 
	contents = cnt;

	recipient_ID_code=recipient_ID_qualifier=recipient_routing_addr=
	recipient_reference=recipient_reference_qualifier=
	interchange_control_reference=process_priority_code=authorization_info=
	authorization_info_qualifier=syntax_ID_string=sender_ID_code=
	sender_ID_qualifier=sender_routing_addr=application_ref = NULL;

	acknowledgement_request=test_indicator = NULL;

	comp_data_element_separator=data_element_separator=decimal_notation=
	release_indicator=reserved=segment_terminator = NULL;

	syntax_version = NULL;
	date_preparation=time_preparation = NULL;
	recipient=edi_message_type=bodypart_type = NULL;

        // Create space for X12 and EDIFACT fields to be filled.
	// Note that some fields have the same name for EDIFACT and X12
	// but have different sizes.  Such fields are created in the 
	// appopriate parse methods.
        authorization_info_qualifier = new char[ AUTH_INFO_QUAL_SIZE ]; 
	assert ( authorization_info_qualifier != NULL );
        authorization_info = new char[ AUTH_INFO_SIZE ]; 
	assert ( authorization_info != NULL );
	// Set the auth info to null for EDIFACT.
	authorization_info[ 0 ] = NULL;
        interchange_control_reference = new char[ INTERCHANGE_CNTL_SIZE ]; 
	assert ( interchange_control_reference != NULL );
        acknowledgement_request = new char[ ACK_REQUEST_SIZE ]; 
	assert ( acknowledgement_request != NULL );
        test_indicator = new char[ TEST_INDICATOR_SIZE ]; 
	assert ( test_indicator != NULL );
        data_element_separator = new char[ ELEMENT_SEPARATOR_SIZE ]; 
	assert ( data_element_separator != NULL );
        comp_data_element_separator = new char[ ELEMENT_SEPARATOR_SIZE ]; 
	assert ( comp_data_element_separator != NULL );
        segment_terminator = new char[ SEGMENT_TERMINATOR_SIZE ]; 
	assert ( segment_terminator != NULL );

        decimal_notation = new char[ DECIMAL_NOTATION_SIZE ]; 
	assert ( decimal_notation != NULL );
        release_indicator = new char[ RELEASE_INDICATOR_SIZE ]; 
	assert ( release_indicator != NULL );
        reserved = new char[ RESERVED_SIZE ]; 
	assert ( reserved != NULL );
        syntax_version = new char[ SYNTAX_VERSION_SIZE ]; 
	assert ( syntax_version != NULL );
        sender_routing_addr = new char[ ROUTING_ADDR_SIZE ]; 
	assert ( sender_routing_addr != NULL );
        recipient_routing_addr = new char[ ROUTING_ADDR_SIZE ];
	assert ( recipient_routing_addr != NULL );
	recipient_reference = new char[ RECIP_REF_SIZE ];
        assert ( recipient_reference != NULL );
	recipient_reference_qualifier = new char[ RECIP_REF_QUALIFIER_SIZE ];
	assert ( recipient_reference_qualifier != NULL );
	application_ref = new char[ APPLICATION_REF_SIZE ];
	assert ( application_ref != NULL );
        process_priority_code = new char[ PROCESS_PRIORITY_CODE_SIZE ];
	assert ( process_priority_code != NULL );
	comm_agreement_ID = new char[ COMM_AGREE_SIZE ];
	assert ( comm_agreement_ID != NULL );

}

//
// ~Interchange()
//
// Class Members:  Interchange
// Parameters:	   
// Called By:	   public/destructor
//
// This method frees the previously allocated space.
//

inline Interchange::~Interchange() {
      
     // delete all previously "new"ed memory.
     if ( data_element_separator != NULL )
	delete data_element_separator;
     if ( comp_data_element_separator != NULL )
     	delete comp_data_element_separator;
     if ( segment_terminator != NULL )
     	delete segment_terminator;
     if ( decimal_notation != NULL )
     	delete decimal_notation;
     if ( release_indicator != NULL )
     	delete release_indicator;
     if ( reserved != NULL )
     	delete reserved;
     if ( syntax_ID_string != NULL )
     	delete syntax_ID_string;
     if ( syntax_version != NULL )
     	delete syntax_version;
     if ( sender_ID_code != NULL )
     	delete sender_ID_code;
     if ( sender_ID_qualifier != NULL )
     	delete sender_ID_qualifier;
     if ( sender_routing_addr != NULL )
     	delete sender_routing_addr;
     if ( recipient_ID_code != NULL )
     	delete recipient_ID_code;
     if ( recipient_ID_qualifier != NULL )
     	delete recipient_ID_qualifier;
     if ( recipient_routing_addr != NULL )
     	delete recipient_routing_addr;
     if ( date_preparation != NULL )
     	delete date_preparation;
     if ( time_preparation != NULL )
     	delete time_preparation;
     if ( interchange_control_reference != NULL )
     	delete interchange_control_reference;
     if ( recipient_reference != NULL )
     	delete recipient_reference;
     if ( recipient_reference_qualifier != NULL )
     	delete recipient_reference_qualifier;
     if ( application_ref != NULL )
    	 delete application_ref;
     if ( process_priority_code != NULL )
     	delete process_priority_code;
     if ( acknowledgement_request != NULL )
     	delete acknowledgement_request;
     if ( comm_agreement_ID != NULL )
     	delete comm_agreement_ID;
     if ( test_indicator != NULL )
     	delete test_indicator;
     if ( bodypart_type != NULL )
	delete bodypart_type;
     if ( edi_message_type != NULL )
	delete edi_message_type;
}

//
// content()
//
// Class Members:  contents
// Called By:	   public
//
// This method returns the address of the interchange string.
//

inline char *Interchange::content() { 
	return contents;
}

//
// get_recipient_ID_code
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method returns the interchange service string advice subelement
// "component data element separtor".
//
inline teletex Interchange::get_recipient_ID_code() {
 
  return (recipient_ID_code);
}

//
// get_recipient_ID_qualifier
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method returns the interchange service string advice subelement
// "component data element separtor".
//
inline teletex Interchange::get_recipient_ID_qualifier() {
 
  return (recipient_ID_qualifier);
}

//
// get_recipient_routing_addr
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method returns the interchange service string advice subelement
// "data element separator".
//
inline teletex Interchange::get_recipient_routing_addr() {
 
  return (recipient_routing_addr);
}

//
// get_recipient_reference
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method returns the interchange service string advice subelement
// "component data element separtor".
//
inline teletex Interchange::get_recipient_reference() {
 
  return (recipient_reference);
}

//
// get_recipient_reference_qualifier
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method returns the interchange service string advice subelement
// "component data element separtor".
//
inline teletex Interchange::get_recipient_reference_qualifier() {
 
  return (recipient_reference_qualifier);
}

//
// get_interchange_control_reference
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method returns the interchange service string advice subelement
// "component data element separtor".
//
inline teletex Interchange::get_interchange_control_reference() {
 
  return (interchange_control_reference);
}

//
// get_process_priority_code
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method returns the interchange service string advice subelement
// "component data element separtor".
//
inline teletex Interchange::get_process_priority_code() {
 
  return (process_priority_code);
}

//
// get_acknowledgement_request
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method returns the interchange service string advice subelement
// "component data element separtor".
//
inline boolean Interchange::get_acknowledgement_request() {
 
  return (acknowledgement_request);
}

//
// get_comm_agreement_ID
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method returns the interchange service string advice subelement
// "component data element separtor".
//
inline teletex Interchange::get_comm_agreement_ID() {
 
  return (comm_agreement_ID);
}

//
// get_test_indicator
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method returns the interchange service string advice subelement
// "component data element separtor".
//
inline boolean Interchange::get_test_indicator() {
 
  return (test_indicator);
}

//
// get_authorization_info
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method returns the interchange service string advice subelement
// "component data element separtor".
//
inline  octet Interchange::get_authorization_info() {
 
  return (authorization_info);
}


//
// get_authorization_info_qualifier
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method returns the interchange service string advice subelement
// "component data element separtor".
//
inline teletex Interchange::get_authorization_info_qualifier() {
 
  return (authorization_info_qualifier);
}

//
// get_comp_data_element_separator
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method returns the interchange service string advice subelement
// "component data element separtor".
//
inline octet Interchange::get_comp_data_element_separator() {
 
  return (comp_data_element_separator);
}

//
// get_data_element_separator
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method returns the interchange service string advice subelement
// "data element separator".
//
inline octet Interchange::get_data_element_separator() {
 
  return (data_element_separator);
}

//
// get_decimal_notation
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method returns the interchange service string advice subelement
// "decimal notation".
//
inline octet Interchange::get_decimal_notation() {
 
  return (decimal_notation);
}

//
// get_release_indicator
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method returns the interchange service string advice subelement
// "release_indicator".
//
inline octet Interchange::get_release_indicator() {
 
  return (release_indicator);
}

//
// get_reserved
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method returns the interchange service string advice subelement
// "reserved".
//
inline octet Interchange::get_reserved() {
 
  return (reserved);
}

//
// get_segment_terminator
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method returns the interchange service string advice subelement
// "segment terminator".
//
inline octet Interchange::get_segment_terminator() {
 
  return (segment_terminator);
}

//
// get_syntax_id_string
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method returns the interchange syntax identifier subfield
// "syntax id string".
//
inline teletex Interchange::get_syntax_id_string() {
 
  return (syntax_ID_string);
}


//
// get_syntax_version
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method returns the interchange syntax identifier subfield
// "syntax version".
//
inline printable Interchange::get_syntax_version() {
 
  return (syntax_version);
}

//
// get_sender_ID_code
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method parses the interchange sender subelement "sender
// identifier code".
//
inline teletex Interchange::get_sender_ID_code() {
 
  return (sender_ID_code);
}

//
// get_sender_ID_qualifier
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method parses the interchange service string advice.
//
inline teletex Interchange::get_sender_ID_qualifier() {
 
  return (sender_ID_qualifier);
}

//
// get_sender_routing_addr
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method parses the interchange service string advice.
//
inline teletex Interchange::get_sender_routing_addr() {
 
  return (sender_routing_addr);
}

//
// get_date_preparation
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method parses the interchange service string advice.
//
inline utc_time Interchange::get_date_preparation() {
 
  return (date_preparation);
}

//
// get_time_preparation
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method parses the interchange service string advice.
//
inline utc_time Interchange::get_time_preparation() {
 
  return (time_preparation);
}

//
// get_application_ref
// 
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method parses the interchange service string advice.
//
inline teletex Interchange::get_application_ref() {
 
  return (application_ref);
}

//
// get_interchange_type
//
// Class Members: Interchange
// Class Methods:
// Parameters: none
// Called By:  parse_interchange(), parse_edi_names()
//
// This method returns the interchange type or none if not supported.
//
inline char *Interchange::get_interchange_type(char *value) {

          bodypart_type = new char[BODYPART_TYPE_SIZE];
	  edi_message_type = new char[EDI_MESSAGE_TYPE_SIZE];
          if (!strncmp(value, "ISA", 3)) { // X12 Interchange?
             // Record the ANSI X12 bodypart type and message type.
             memcpy( bodypart_type, OMP_O_EDI_BPO_ANSIX12_ISO646, 5 );
	     bodypart_type[ 5 ] = '\0';
             strcpy( edi_message_type, "840" ) ;
             return("ISA"); // X12 Interchange.
          }
          else if (!strncmp(value, "UNA", 3)) { // EDIFACT Interchange?
             // Record the EDIFACT bodypart type.
             memcpy( bodypart_type, OMP_O_EDI_BPO_EDIFACT_ISO646, 5 );
	     bodypart_type[ 5 ] = '\0';
             strcpy( edi_message_type, "PURORD" );
             return("UNA"); \
                }
	        else return ("NOT");// Interchange type NOT supported.

}


//
// get_bodypart_type
//
// Class Members: Interchange
// Class Methods:
// Parameters: none
// Called By:  transmit_interchange
//
// This method returns the bodypart type which was previously
// determined (while parsing the interchange).
//
inline char *Interchange::get_bodypart_type() {
  return ( bodypart_type );
}

//
// get_edi_message_type
//
// Class Members: Interchange
// Class Methods:
// Parameters: none
// Called By: 
//
// This method returns the message type which was previously
// determined (while parsing the interchange).
//
inline char *Interchange::get_edi_message_type() {
  return ( edi_message_type );
}



