//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//                                                                      //
//      FILE NAME : x500.h												//
//                                                                      //
//      DESCRIPTION :                                                   //
//		This file contains class, type and constant definitions for use //
//		by the X.500 i/f of the EDI Communications Platform software.	//
//                                                                      //
//      BUGS, WARNINGS AND SPECIAL COMMENTS :                           //
//		Be careful, it might bite you!									//
//                                                                      //
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

// The EDI Object Class definitions make use of various attributes defined 
// in the EDI messaging standard (CCITT X.435). Some of these attributes
// have syntax BIT STRING, but the BIT STRINGs are never more than 3 bits
// in length. The following masks are defined in order to readily decode
// the BIT STRING values from the OM_Values received from the X.500 API
// (see IEEE P1327/D6 section 3.2.13 for description of the representation
// of the BIT STRING by the API).

void tellme( char*, char* );

#define BIT_ZERO	0x40
#define BIT_ONE		0x20
#define BIT_TWO		0x10

class X500Entry {

	//
	// Attributes common to all transmission media.
	//
	
	PROT_ID		protocol_id;
	char		*send_id_code;
	char		*send_id_code_qualifier;
	char		*rcpt_id_code;
	char		*rcpt_id_code_qualifier;
	
	// 
	// X.400 Attributes.
	//

	ORName		*this_recip;
	ORName		*edin_receiver;
//	HP X.500 cannot handle enumerated attribute syntax, so make orig-security
//	an integer
//	SECURITY	orig_security;
	int			orig_security;
	BOOL		positive_notif;
	BOOL		negative_notif;
	BOOL		forwarded_notif;
	BOOL		proof_of_notif;
	BOOL		nonrepud_of_notif;
	BOOL		proof_of_rcpt;
	BOOL		nonrepud_of_rcpt;
	BOOL		resp_passing;
//	HP X.500 cannot handle enumerated attribute syntax, so make prior
//	an integer
//	OM_enumeration	prior;
	int			prior;

	// 
	// FTAM Attributes.  Change the constraints as appropriate!!!!
	//

	char		name[ MAX_LINE ];
	char		password[ MAX_LINE ];
	char		x500_name[ MAX_LINE ];
	char		dir_name[ MAX_LINE ];

	// 
	// VAN Attributes.  Change the constraints as appropriate!!!!
	//

	char		vanname[ MAX_LINE ];

	//
	// Private function declarations
	//

	STATUS mapMhsEdiAttrs( OM_public_object, OM_value_position );
	STATUS mapMhsIpmAttrs( OM_public_object, OM_value_position );
	STATUS mapFtamAttrs( OM_public_object, OM_value_position );
	STATUS mapVanAttrs( OM_public_object, OM_value_position );

	
public:

	X500Entry::X500Entry() {

		protocol_id = MHS_EDI;
		send_id_code=send_id_code_qualifier = NULL;
		rcpt_id_code=rcpt_id_code_qualifier = NULL;
		this_recip = edin_receiver = NULL;
		positive_notif=negative_notif=forwarded_notif = FALSE;
		proof_of_notif=nonrepud_of_notif = FALSE;
		proof_of_rcpt=nonrepud_of_rcpt = FALSE;
		resp_passing = FALSE;
		prior = (int)MH_PTY_NORMAL;
		orig_security = (int)NO_SECURITY;
	}

	PROT_ID	  transmission_medium()	           { return protocol_id;       }
	
	//
	// Functions to return X.400 Attributes.
	//
	
	ORName	  *this_recipient() 	           { return this_recip;        }
	ORName	  *edin_receiver_name()	           { return edin_receiver;     }
//	HP X.500 cannot handle enumerated attribute syntax, so make orig-security
//	an integer
	int  	  origination_security()	   { return orig_security;     }
	BOOL	  positive_notification()	   { return positive_notif;    }
	BOOL	  negative_notification()	   { return negative_notif;    }
	BOOL	  forwarded_notification()         { return forwarded_notif;   }
	BOOL	  nonrepudiation_of_notification() { return nonrepud_of_notif; }
	BOOL	  proof_of_notification()          { return proof_of_notif;    }
	BOOL	  nonrepudiation_of_reception()    { return nonrepud_of_rcpt;  }
	BOOL	  proof_of_reception()             { return proof_of_rcpt;     }
	BOOL	  resp_passing_allowed()	   { return resp_passing;      }
	OM_enumeration	priority()	           { return prior; 	       }
	
	//
	// Function to return FTAM Attributes.
	//
	
	char	*user_name()			{ return name;		}
	char	*user_password()		{ return password;	}
	char	*x500_dirname()			{ return x500_name;	}
	char	*file_dirname()			{ return dir_name;	}

	//
	// Function to return VAN Attributes.
	//
	
	char	*van_name()			{ return vanname;	}
	
	
	//
	// Query the Directory.
	//
	
	STATUS read_communication_attributes( char *, char *, char *, char * );
	STATUS validateTPByORName( char *, char *, ORName * );
	STATUS validateTPByORName( char *, char *, OM_public_object );
	STATUS validateTPByLocalName( char *, char *, char *, char * );
	void printEntry();

};


//
// Define the elements of the superior entry for all trading partner entries
//

#define EDI_PARTNER_COUNTRY			"US"
#define EDI_PARTNER_ORGANIZATION	"NIST"
#define EDI_PARTNER_ORG_UNIT		"EDI"

/*
 * declare an error checking macro for DS
 */

#define CHECK_DS_CALL(function_call)			\
		error = ( (DS_status) function_call);	\
		if (error != DS_SUCCESS) {				\
			tellme( myname, "Bad DS function return\n" );\
			at_print_object( (OM_object)error ); \
			om_delete( ( OM_object ) error );	\
		}
//			cerr << "Bad DS function return.";  \

/*
 * declare an error checking macro for OM
 */

#define CHECK_OM_CALL(function_call)		\
		return_code = (function_call);	\
		if (return_code != OM_SUCCESS)	\
			handle_om_error(return_code);

//
// Macro to replace the API macro OM_STRING(), whose functionality is not
// supported by the HP compiler.
// eg "CC: sorry, not implemented: general initializer in  initializer list"
//
#define LOCAL_OM_STRING( om_string, string )							\
		om_string.length = (OM_string_length)(sizeof( string ) - 1);	\
		om_string.elements = (void *)string;
