//				FILE = edi_out_master.C

//	This program is the master program that spawns slave programs
//	to transfer EDI transactions to a remote host.  This program
//	will periodically check three directories to determine if there
//	are any transactions to be sent.  Depending on the type of
//	transfer defined (FTAM, X.25 or XMODEM) the appropriate transfer
//	slave program will be executed.  There will be three copies of this
//	program executing simultaneously (each operating on a set of
//	different directories); one for FTAM, one for X.25 and one for
//	XMODEM.
//
//	This program can be stopped gracefully by having the edi_daemon
//	program sent a SHUTDOWN command via the ipc message facility
//	(ipc_msg).  An ABORT command will immediately stop the program.

// using the -D option on the compile command define one of the following
// three transfer types: FTAM_OUT, XMODEM_OUT, or X25_OUT

#undef	FTAM_IN
#undef	XMODEM_IN
#undef	X25_IN

#include	"xmodem.h"
#include	"xmodem_allocate.h"
#include	"msg_edi.h"
#include	"msg_defs.h"
#include	"functions.h"
#include	"edi_control_defs.h"
#include	"edi_transfer_defs.h"

ipc_msg	edi_out((key_t) KEY_EDI,(long) TRANSFER_TYPE,(int) MSG_LONG_TEXT);

itimerval	edi_itime = {INTERVAL_SECONDS,0,INTERVAL_SECONDS,0};

void	sig_handler(int);

struct	sigvec vec = {(void (*)(...))sig_handler, 0, 0};
int	sigalrm = FALSE;

main(int argc, char *argv[])
{
	int	len, num, i, j, num_x, num_c, error_count = 0, sleep_time;
	int	fd = -1, fd_c = -1, fd_x = -1, fd_l = -1;
	int	defaults = DEFAULTS, rm_log = TRUE;
	struct	dirent	*p, *p1[256];
	char	name_x[MAX_FILENAME], name_c[MAX_FILENAME];
	char	name_l[MAX_FILENAME], unique_id[65], startup[MAX_FILENAME];
	char	*xaction_dir, *control_dir, *lock_dir;
	char	lock_file[MAX_FILENAME];
	char	config[MAX_CONTROL_FILE_SIZE];
	char	*config_strings[MAX_CONTROL_STRINGS + 1];
	char	buf_dir[MAX_DIR_SIZE];
	char	*ptr1, *ptr2, *ptr3, *slave_argv[SLAVE_ARGVS];
	char	*ptr_x, *ptr_c, *ptr_l, **ptr;
	DIR	*dirp_c, *dirp_x;
	char	*ptr_s = STARTUP_FILENAME;


	if ((argc == 2) && (strcmp(argv[1],"help") == 0))	{
		logit(USAGE);
		exit(0);
	}

	if (get_log(argv[0]) != TRUE)	{
		sprintf(log_str,"%s: Unable to get log file - exit\n", argv[0]);
		logit(log_str);
		exit(0);
	}

	// set the lock so that only one copy of this program can be
	// running at a time - reset automatically upon exit
	if ((i = prog_exists(TRANSFER_TYPE,0)) == TRUE)	{
		sprintf(log_str,"%s already running - exit\n", argv[0]);
		logit(log_str);
		exit(0);
	}

	// for KEEP/DELETE_LOG use here as well as pass to slave process
	for (i = 1; i <= argc; i++)	{
		if (strcmp(argv[i],"KEEP_LOG") == 0)	{
			rm_log = FALSE;
			continue;
		}
		if (strcmp(argv[i],"DELETE_LOG") == 0)	{
			rm_log = TRUE;
			continue;
		}
	}

	// retrieve names for xaction, control, and lock directories
	if ((fd = open(PLATFORM_CFG,O_RDONLY)) < 0)	{
		if (defaults == FALSE)	{
			sprintf(log_str,"%s: Configuration file does not exist - abort\n",TRANSFER_MODE);
			logit(log_str);
			exit(1);
		}
		// configuration file does not exist use defaults
		xaction_dir = OUTPUT_XACTION_DIR;
		control_dir = OUTPUT_CONTROL_DIR;
		lock_dir = OUTPUT_LOCK_DIR;
	}	else
		if ((len = read(fd,config,MAX_CONTROL_FILE_SIZE)) < 0)	{
			close(fd);
			logit("Error reading platform configuration file - abort\n");
			exit(1);
		}	else	{
			close(fd);
			if ((num = lines_to_strings(config, len, config_strings, MAX_CONTROL_STRINGS)) <= 0)	{
				logit("Error converting configuration file - abort\n");
				exit(1);
			}	else	{
				if ((num > XACTION_DIR_INDEX) && (XACTION_DIR_INDEX >= 0))
					xaction_dir = config_strings [XACTION_DIR_INDEX];
				else	if (defaults == FALSE)	{
						sprintf(log_str,"%s: No transaction directory - abort\n",TRANSFER_MODE);
						logit(log_str);
						exit(1);
					}	else
						xaction_dir = OUTPUT_XACTION_DIR;
				if ((num > CONTROL_DIR_INDEX) && (CONTROL_DIR_INDEX >= 0))
					control_dir = config_strings [CONTROL_DIR_INDEX];
				else	if (defaults == FALSE)	{
						sprintf(log_str,"%s: No control directory - abort\n",TRANSFER_MODE);
						logit(log_str);
						exit(1);
					}	else
						control_dir = OUTPUT_CONTROL_DIR;
				if ((num > LOCK_DIR_INDEX) && (LOCK_DIR_INDEX >= 0))
					lock_dir = config_strings [LOCK_DIR_INDEX];
				else	if (defaults == FALSE)	{
						sprintf(log_str,"%s: No lock directory - abort\n",TRANSFER_MODE);
						logit(log_str);
						exit(1);
					}	else
						lock_dir = OUTPUT_LOCK_DIR;
			}
		}

	// directory name can not be null
	if (!lock_dir || !*lock_dir || !control_dir || !*control_dir || !xaction_dir || !*xaction_dir)	{
		sprintf(log_str,"%s: Null directory name not allowed - abort\n",TRANSFER_MODE);
		logit(log_str);
		exit(1);
	}

	// create lock filename
	strcpy(lock_file,lock_dir);
	strcat(lock_file,"/");
	strcat(lock_file,LOCK_FILE);

	// partially initialize slave process parameter values
	strcpy(name_x,PFILE_DEF);
	ptr_x = name_x + sizeof(PFILE_DEF) -1;
	strcpy(ptr_x,xaction_dir);
	strcat(ptr_x,"/");
	ptr1 = ptr_x + strlen(ptr_x);
	strcpy(name_c,CTRL_DEF);
	ptr_c = name_c+ sizeof(CTRL_DEF) -1;
	strcpy(ptr_c,control_dir);
	strcat(ptr_c,"/");
	ptr2 = ptr_c + strlen(ptr_c);
	strcpy(name_l,LOCK_DEF);
	ptr_l = name_l + sizeof(LOCK_DEF) -1;
	strcpy(ptr_l,lock_dir);
	strcat(ptr_l,"/");
	ptr3 = ptr_l + strlen(ptr_l);
	strcpy(unique_id,UNIQUE_ID_DEF);

	// initialize slave process parameter addresses
	slave_argv[0] = SLAVE_PROCESS;
	slave_argv[1] = name_x;
	slave_argv[2] = name_c;
	slave_argv[3] = name_l;
	slave_argv[4] = unique_id;
	for (i = 5, j = 1; argc > j; j++, i++)
		slave_argv[i] = argv[j];
	for (ptr = parms; (ptr != 0) && (*ptr != 0); ptr++, i++)
			slave_argv[i] = *ptr;

	if ((ptr_s != 0) && (*ptr_s != '\0'))	{
		strcpy(startup,STARTUP_DEF);
		ptr_s = startup + sizeof(STARTUP_DEF) -1;
		strcat(startup,control_dir);
		if (control_dir[strlen(control_dir) -1] != '/')
			strcat(startup,"/");
		strcat(startup,STARTUP_FILENAME);
		// if file exists add to parameter list
		if ((fd = open(ptr_s, O_RDONLY)) >= 0)	{
			close(fd);
			fd = -1;
			slave_argv[i++] = startup;
		}
	}

	slave_argv[i] = 0;

	// initialize interval timer
	if (setitimer(ITIMER_REAL,&edi_itime, 0) == -1)	{
		logit("Unable to initialize interval timer\n");
		perror(TRANSFER_MODE);
		exit(1);
	}

	// set to handle the SIGALRM signal from setitimer
	sigvector((int) SIGALRM, &vec, 0);
	// set to handle the SIGCHLD signal when spawned processes die
	sigvector((int) SIGCHLD, &vec, 0);

	// check if class ipc_msg object successfully initialized
	if (!edi_out.msg_get_text())	{
		logit("Unable to initialize message facility - exit\n");
		exit(1);
	}

	// THE REST OF THIS PROGRAM IS AN INFINITE LOOP TO
	// CONTINUOUSLY PROCESS MESSAGES AND EDI OUTPUT FILES

	while (1)	{	// BEGIN-INFINITE-LOOP
	// check for messages, and process them all before continuing
	while (edi_out.msg_rcv(TRANSFER_TYPE))	{
		logit("IPC message received\n");
		edi_out.msg_print_cmd();
		if (edi_out.msg_get_cmd() == ABORT)	exit(1);
		if (edi_out.msg_get_cmd() == SHUTDOWN)	{
			// sub_cmd has shutdown time to wait
			sleep_time = edi_out.msg_get_sub_cmd();
			if (sleep_time <= 0)
				sleep_time = COMPLETION_TIME;
			release_log(rm_log);
			// allow children to complete
			sleep(sleep_time);
			exit(0);
		}
		if (edi_out.msg_get_cmd() == COMMAND)	{
			logit("COMMAND not implemented, skipped\n");
			continue;
		}
			sprintf(log_str,"Unknown message command (%d. : '%s') - ignored\n",
				edi_out.msg_get_cmd(), edi_out.msg_get_text());
			logit(log_str);
	}

	// create exclusive use directory lock file - only one process
	// may access files in 'xaction' and 'control' directories at a
	// time	- don't wait forever, bump error count and continue
	for (i = 0; i < WAIT_COUNT; i++)	{
		if ((fd_l = open(lock_file,
				(O_CREAT | O_RDWR | O_EXCL), 0644)) > 0)
			goto	locked;
		sleep(WAIT_INTERVAL);
	}
	sprintf(log_str,"unable to access '%s'\n",lock_file);
	logit(log_str);
	error_count++;
	goto	fini;

	// now own the directory lock file - put program name in file
locked:	write(fd_l,argv[0],strlen(argv[0]));
	close(fd_l);
	fd_l = -1;

	// open control_dir to get 'control' file names
	if ((dirp_c = opendir(control_dir)) == 0)	{
		sprintf(log_str, "Unable to open %s directory.\n", OUTPUT_CONTROL_DIR);
		logit(log_str);
		perror(TRANSFER_MODE);
		error_count++;
		goto	fini;
	}


	// check for orphaned 'control' files - each 'control' file
	// must have a corresponding 'xaction' file - delete orphans
	//
	// skip '.' and '..' files
	// num_c = # of 'control' directory entries to process
	num_c = 0;
	while (p = readdir(dirp_c))	{
		// read control file names from control_dir
		if (strcmp(p->d_name,".") == 0)		continue;
		if (strcmp(p->d_name,"..") == 0)	continue;
		if (strcmp(p->d_name,STARTUP_FILENAME) == 0)	continue;
		// make full file names for 'xaction',
		// 'control', and 'lock' files
		*ptr1 = *ptr2 = *ptr3 = '\0';
		strcat(ptr1,p->d_name);
		strcat(ptr2,p->d_name);
		strcat(ptr3,p->d_name);
		// if corresponding 'lock' file exits,
		// 'control' and 'xaction files busy
		if ((fd_l = open(ptr_l, O_RDONLY)) > 0) {
			close(fd_l);
			continue;
		}
		// 'xaction' file corresponding to
		// 'control' file must exist or error
		if ((fd_x = open(ptr_x,O_RDONLY)) < 0) {
			unlink(ptr_c);
			sprintf(log_str,"control file '%s' deleted, no transaction file\n", ptr_c);
			logit(log_str);
		}	else	{
			close(fd_x);
			p1[num_c] = p;	/* debug */
			num_c++;
		}
	}
	closedir(dirp_c);

/* debug sprintf(log_str,"num_c = %d. dir = '%s'\n",num_c,control_dir); */
/* debug logit(log_str); */
/* debug */	for (i = 0; i < num_c; i++)	p_dirent(p1[i]);

	// open xaction_dir to get 'xaction' file names
	if ((dirp_x = opendir(xaction_dir)),0)	{
		sprintf(log_str,"Unable to open %s directory.\n", xaction_dir);
		logit(log_str);
		perror(TRANSFER_MODE);
		error_count++;
		goto	fini;
	}

	// check for orhaned 'xaction' files - each 'xaction' file must
	// have corresponding 'control' file - delete orphans
	//
	// spawn process to transfer EDI interaction - will delete
	// corresponding 'xaction' and 'control' files
	//
	// skip '.' and '..' files
	// num_x = # of 'xaction' directory entries processed
	num_x = 0;
	while (p = readdir(dirp_x))	{
		// read transaction file names from xaction_dir
		if (strcmp(p->d_name,".")&&strcmp(p->d_name,".."))	{
			// make full file names for 'xaction',
			// 'control', and 'lock' files
			*ptr1 = *ptr2 = *ptr3 = '\0';
			strcat(ptr1,p->d_name);
			strcat(ptr2,p->d_name);
			strcat(ptr3,p->d_name);
			// create corresponding 'lock' file,
			// locks 'control' and 'xaction' files;
			// if 'lock' file exists, 'xaction' and
			// 'control' files are busy
			if ((fd_l = open(ptr_l,
				(O_RDWR | O_CREAT | O_EXCL), 0666)) < 0)
					continue;
			close(fd_l);
			// open corresponding 'control' file
			if ((fd_c = open(ptr_c,O_RDONLY)) < 0) {
				unlink(ptr_x);
				unlink(ptr_l);
				sprintf(log_str,"transaction file '%s' deleted, no control file\n", ptr_x);
				logit(log_str);
			}	else	{
				close(fd_c);
				p1[num_x] = p;	/* debug */
				num_x++;
				// add unique_id to save parameter
				strcpy(&unique_id[sizeof(UNIQUE_ID_DEF)- 1], make_unique());
				// spawn a slave process to transmit EDI
				// interchange - will delete 'xaction',
				// 'control', and 'lock' files
				spawn(SLAVE_PROCESS,&slave_argv[0]);
			}
		}
	}
	closedir(dirp_x);

/* debug sprintf(log_str,"num_x = %d. dir = '%s'\n",num_x,xaction_dir); */
/* debug logit(log_str); */
/* debug for (i = 0; i < num_x; i++)	p_dirent(p1[i]); */

	if (num_x != num_c)	{
		sprintf(log_str,"Orphan check failed: num_x = %d num_c = %d\n",
			num_x,num_c);
		logit(log_str);
	}
fini:	// unlink the directory lock file so that files may be created
	unlink(lock_file);
	if (error_count > MAX_ERROR_COUNT)	exit(1);

	// wait for interval timer, ignore multiple expirations
	while (sigalrm == FALSE)	sigpause(0);
	sigalrm = FALSE;
	}	//	END-INFINITE-LOOP
	logit("broke out of inifinte loop ???\n");
}

FUNCTION p_dirent(struct dirent *p)
{
	sprintf(log_str,"d_fileno = %d. d_reclen = %d. d_namlen = %d.\n",
		p->d_fileno,p->d_reclen,p->d_namlen);
	logit(log_str);
	sprintf(log_str,"d_name = '%s'\n",p->d_name);
	logit(log_str);
	return(TRUE);
}

//	Spawn will 'vfork' and 'execv' a new process to process the 
//	outgoing EDI transaction.  
FUNCTION spawn(char *process, char **argv)
{
	pid_t pid;
/* debug */	char	**ptr = argv;
/* debug */	sprintf(log_str,"process = '%s'\n",process);
/* debug */	logit(log_str);
/* debug */	while (*ptr)	{
/* debug */		sprintf(log_str,"argv = '%s'\n",*ptr++);
/* debug */		logit(log_str);
/* debug */	}
	pid = vfork();
	if (pid != 0)	return(pid);
	execv(process,argv);
	perror("spawn");
	exit(1);
	return(TRUE);
}

FUNCTION void sig_handler(int sig)
{
	if (sig == SIGCHLD)	{
		wait(0);
		// restore signal handler for death of child
		sigvector((int) SIGCHLD, &vec, 0);
	}	else
		if (sig == SIGALRM)
			// set to handle the SIGALRM signal from setitimer
			sigalrm = TRUE;
		else	{
			sprintf(log_str,"sig_handler: Signal -%d.- ignored\n",sig);
			logit(log_str);
		}
}

FUNCTION char *make_unique()
{
	static	char	unique[MSG_LONG_TEXT];
	static	long	pid = 0;
	char	*p;
	if (pid == 0)	pid = getpid();
	strcpy(unique,TRANSFER_MODE);
	p = unique + sizeof(TRANSFER_MODE) -1;
	strcpy(p++,":");
	ltoa(p,time(0));
	p = unique + strlen(unique);
	strcpy(p++,"/");
	ltoa(p,pid);
	return(unique);
}

FUNCTION void ltoa(char *to, long num)
{
	char	temp[32], *str;
	for (str = temp; num ; num /= 10)
		*str++ = '0' + (char)(num % 10);
	while (str != temp)
		*to++ = *--str;
	*to = '\0';
}
