//				FILE = edi_in_master.C

//	This program is the master program that spawns slave programs
//	to transfer EDI transactions from a remote hosts.  This program
//	will periodically check a file (specified  by platform.cfg) to
//	determine if there are any hosts to attempt to receive transactions
//	from and at what time to try to retrieve them.  This program is run
//	from the user crontab file.  It scans the list of hosts and
//	determines if it is time to attempt a transfer.  For each remote
//	host for which it is time a slave process is spawned.  When the list
//	is exhausted the process terminates until cron runs it again.  There
//	are three copies of this program, one for each transfer type.
//
//	The program will check a lock file to determine if transfers are
//	allowed.  If transfers are not allowed the program will exit.

// using the -D option on the compile command define one of the following
// three transfer types: FTAM_IN, XMODEM_IN, or X25_IN


#undef	X25_OUT
#undef	XMODEM_OUT
#undef	FTAM_OUT

#define	CHILD_WAIT_TIME	3600

#include	"msg_edi.h"
#include	"msg_defs.h"
#include	"functions.h"
#include	"edi_control_defs.h"
#include	"edi_transfer_defs.h"

void    sig_handler(int);

struct  sigvec vec = {(void (*)(...))sig_handler, 0, 0};
int     sigalrm = FALSE;

main(int argc, char *argv[])
{
	int	hour,len,num,i,j,num_h = 0,fd,fd_c,defaults = DEFAULTS;
	int	index, nparms, protocol, err_index, result = FALSE;
	char	name_h[MAX_HOSTNAME + 1], *input_host_file = 0;
	char	*xaction_dir = 0, config[MAX_CONTROL_FILE_SIZE];
	char	*config_strings[MAX_CONTROL_STRINGS + 1];
	char	platform[MAX_CONTROL_FILE_SIZE], name_x[MAX_FILENAME +1];
	char	*platform_strings[MAX_CONTROL_STRINGS + 1];
	char	*p1, *p2, *p3, *p4, *slave_argv[SLAVE_ARGVS];
	char	*ptr_h, *ptr_c, name_c[MAX_FILENAME + 1], *p, **ptr;
	char	*ctrl_dir = 0, *control_dir = 0, *ctrl_p = 0;
	char	*ctrl_file = 0, *control_file = 0, *blockfile = 0;
	struct	tm *tm_ptr;
	struct	timeval	timeval;
	struct	timezone timezone;
	long	clock;


	if ((argc == 2) && (strcmp(argv[1],"help") == 0))	{
		logit("Usage: edi_in_master [options]\n\twhere 'options' are passed to the transfer program xmodem_copy\n");
		exit(0);
	}

	if (get_log(argv[0]) != TRUE)	{
		logit("edi_in_master: Unable to get log file - abort\n");
		exit(1);
	}

	if (defaults)	{
		blockfile = XMODEM_INPUT_LOCK_FILE;
		ctrl_p = DEFAULT_INPUT_CTRL_FILE;
	}

	// retrieve name of directory to receive transactions and
	// file of remote host names and retrieval times
	if ((fd = open(PLATFORM_CFG,O_RDONLY)) < 0)	{
		logit("File platform.cfg does not exist - abort\n");
		exit(1);
	}	else
		if ((len = read(fd,platform,MAX_CONTROL_FILE_SIZE)) < 0)	{
			close(fd);
			logit("Error reading platform configuration file - abort\n");
			exit(1);
		}	else	{
			close(fd);
			if ((num = lines_to_strings(platform, len, platform_strings, MAX_CONTROL_STRINGS)) <= 0)	{
				logit("Error converting configuration file - abort\n");
				exit(1);
			}	else	{
				if ((num > INPUT_DIR_INDEX) && (INPUT_DIR_INDEX >= 0))
					xaction_dir = platform_strings[INPUT_DIR_INDEX];
				else	if (defaults)
						xaction_dir = INPUT_XACTION_DIR;
					else	{
						sprintf(log_str,"%s: No input directory - abort\n",TRANSFER_MODE);
						logit(log_str);
						exit(1);
					}
				if ((num > INPUT_HOST_INDEX) && (INPUT_HOST_INDEX >= 0))
					input_host_file = platform_strings[INPUT_HOST_INDEX];
				else	if (defaults)
						input_host_file = INPUT_HOST_FILE;
					else	{
						sprintf(log_str,"%s: No host directory - abort\n",TRANSFER_MODE);
						logit(log_str);
						exit(1);
					}
				if ((num > INPUT_BLOCK_INDEX) && (INPUT_BLOCK_INDEX >= 0))
					blockfile = platform_strings[INPUT_BLOCK_INDEX];
			}
		}

	// list of remote xmodem input hosts must be specified
	if ((input_host_file == 0) || (*input_host_file == 0))	{
		logit("No remote hosts specified\n");
		exit(0);
	}

	// if file exists no input transfers are allowed
	if (blockfile && (fd = open(blockfile,O_RDONLY)) >= 0)	{
		close(fd);
		sprintf(log_str,"blockfile - '%s' - exists, exit\n", blockfile);
		logit(log_str);
		exit(0);
	}

	// partially initialize slave process parameter values
	strcpy(name_h,HOST_DEF);
	ptr_h = name_h + sizeof(HOST_DEF) -1;
	strcpy(name_c,CTRL_DEF);
	ptr_c = name_c + sizeof(CTRL_DEF) -1;
	strcpy(name_x,GDIR_DEF);
	strcat(name_x,xaction_dir);

	// initialize slave process parameter addresses
	slave_argv[0] = SLAVE_PROCESS;
	slave_argv[1] = name_c;
	slave_argv[2] = name_x;
	for (index = 3, j = 1; argc > j; j++, index++)
		slave_argv[index] = argv[j];
	for (ptr = parms; (ptr != 0) && (*ptr != 0); ptr++, index++)
			slave_argv[index] = *ptr;
	slave_argv[index] = 0;
	nparms = index;

	// open input_host_file to get host names
	if ((fd = open(input_host_file,O_RDONLY)) < 0)	{
		err_index = errno;
		sprintf(log_str,"Unable to open host file list - %s.\n", input_host_file);
		logit(log_str);
		sprintf(log_str,"%s: %s\n",TRANSFER_MODE, sys_errlist[err_index]);
		logit(log_str);
		exit(1);
	}

	// Read input_host_file to get remote host name, (optionally)
	// control file name, and list of retrieval times for each host.
	len = read(fd,config,MAX_CONTROL_FILE_SIZE);
	close(fd);
	if (len <= 0)	{
		sprintf(log_str,"Unable to read host name file - '%s'\n", input_host_file);
		logit(log_str);
		exit(1);
	}

	// convert text lines to null terminated strings
	if (lines_to_strings(config,len,config_strings,MAX_CONTROL_STRINGS) <= 0)	{
		sprintf(log_str,"Unable to convert remote host name file to strings - '%s'\n", input_host_file);
		logit(log_str);
		exit(1);
	}

	// catch signal when child dies
	sigvector((int) SIGCHLD, &vec, 0);

	// get current hour
	gettimeofday(&timeval,&timezone);
	clock = timeval.tv_sec;
	tm_ptr = localtime(&clock);
	hour = tm_ptr->tm_hour;

	// Process remote host name file one line at a time.
	// If time, spawn process to retrieve EDI interaction.
	//
	// num_h = # of host entries processed
	for (ptr = config_strings; *ptr != 0; ptr++)	{ // get next entry
		p = *ptr;
		// find start of record
		while (WHITE(*p))	p++;
		if (*p == '\0')	continue;	// skip null record
		*ptr_c = '\0';
sprintf(log_str,"Processing record = '%s'\n",p);
logit(log_str);
		if (strncmp(p,CTRL_DEF,sizeof(CTRL_DEF) -1) == 0)	{
			// control file directive
			if (strcmp(p+sizeof(CTRL_DEF)-1,"default") == 0)	{
				control_dir = 0;
				if (defaults)
					control_file = DEFAULT_INPUT_CTRL_FILE;
				else	control_file = 0;
			}	else
				if ((*(p+sizeof(CTRL_DEF)-1)) == '\0')	{
					control_file = 0;
					control_dir = 0;
				}	else
					if ((*(p+strlen(p)-1)) == '/')	{
						control_dir = p + sizeof(CTRL_DEF)-1;
						control_file = 0;
					}	else	{
						control_dir = 0;
						control_file = p + sizeof(CTRL_DEF)-1;
					}
			continue;
		}

		// check host name
		p1 = p2 = p;
		while(!WHITE(*p2) && (*p2 != '\0'))	p2++;
		if (*p2 == '\0')	{
			p3 = p2;
		}	else	{
			*p2 = '\0';
			p3 = p2 + 1;
		}
		// '*' => host name in control file, skip
		if (strcmp(p1,"*") != 0)	{
			// add to parameter list
			slave_argv[index++] = name_h;
			strcpy(ptr_h,p1);	// copy host name
			slave_argv[index] = 0;
		}
		if (p2 != p3)	*p2 = ' ';	// restore "original" record

		// make control file name
		p1 = p3;
		while (WHITE(*p1))	p1++;
		p2 = p1;
		while (!WHITE(*p2) && (*p2 != '\0'))	p2++;
		if (*p2 == '\0')	{
			p3 = p2;
		}	else	{
			*p2 = '\0';
			p3 = p2 + 1;
		}
		if (*p1 == '*')	{
			ctrl_dir = 0;
			ctrl_file = 0;
		}
		else	if (strcmp(p1,"default") == 0)	{
				ctrl_dir = 0;
				if (defaults)
					ctrl_file = DEFAULT_INPUT_CTRL_FILE;
				else	ctrl_file = 0;
			}	else
				if (*(p2-1) == '/')	{
					ctrl_dir = p1;
					ctrl_file = 0;
				}	else	{
					ctrl_dir = 0;
					if (*p1 == 0)
						ctrl_file = 0;
					else
						ctrl_file = p1;
				}
		if ((ctrl_file == 0) && (control_file == 0) && (ctrl_p == 0))	{
			sprintf(log_str,"No control file specified, record skipped - '%s'\n", *ptr);
			logit(log_str);
			result = FALSE;
			continue;
		}

		// add the control directory (if it exists) to the
		// parameter to the spawn process
		if (ctrl_dir != 0)	strcpy(ptr_c,ctrl_dir);
		else	if ((control_dir != 0) && (*ctrl_file != '/'))
				strcpy(ptr_c,control_dir);

		// add the control file name to the parameter
		if (ctrl_file != 0)	strcat(ptr_c,ctrl_file);
		else	if (control_file != 0)
				strcat(ptr_c,control_file);
			else	if (ctrl_p != 0)
					strcat(ptr_c,ctrl_p);
		if (p2 != p3)	*p2 = ' ';	// restore "original" record

		// check the time parameter - none or '*' means everytime
		p1 = p3;
		while (WHITE(*p1))	p1++;
		p2 = p1;
		while (!WHITE(*p2) && (*p2 != '\0'))	p2++;
		if (*p2 == '\0')	{
			p3 = p2;
		}	else	{
			*p2 = '\0';
			p3 = p2 + 1;
		}
		if (*p1 == '-')	goto	next;	// temporarily blocked
		if ((*p1 != '\0') && (*p1 != '*'))
			// check list of hours in remote host entry
			while (TRUE)	{
				p4 = p1;
				if (!isdigit(*p1))	{
					if (p2 != p3)	*p2 = ' '; // restore record
					sprintf(log_str,"record = '%s'\n", *ptr);
					logit(log_str);
					sprintf(log_str, "	Time value not numeric, record skipped - '%s'\n",p1);
					logit(log_str);
					result = FALSE;
					goto	next;
				}
				while (isdigit(*p1))	p1++;
				if ((*p1 != ',') && (*p1 != '\0'))	{
					if (p2 != p3)	*p2 = ' '; // restore record
					sprintf(log_str,"record = '%s'\n", *ptr);
					logit(log_str);
					sprintf(log_str, "	Invalid character in time field, record skipped - '%s'\n",p1);
					logit(log_str);
					result = FALSE;
					goto	next;
				}
				*p1 = '\0';
				i = -1;
				sscanf(p4,"%d",&i);
				if (i == -1)	goto	next;
				if (i == hour)	break;	// time matches
				if (p1 != p3)	*p1 = ' '; // restore record
				p1++;
				if (p1 >= p2)	goto	next;
			}
		if (p2 != p3)	*p2 = ' ';	// restore "original" record

		// check protocol field, if not this protocol then skip
		p1 = p3;
		while (WHITE(*p1))	p1++;
		p2 = p1;
		while (!WHITE(*p2) && (*p2 != '\0'))	p2++;
		if (*p2 == '\0')	{
			p3 = p2;
		}	else	{
			*p2 = '\0';
			p3 = p2 + 1;
		}
		if (*p1 == '\0')	protocol = DEFAULT_INPUT_PROTOCOL;
		else	if (strcmp(p1,"XMODEM") == 0)
				protocol = XMODEM_PROTOCOL;
		else	if (strcmp(p1,"FTAM") == 0)
				protocol = FTAM_PROTOCOL;
		else	if (strcmp(p1,"X25") == 0)
				protocol = X25_PROTOCOL;
		else	{
			if (p2 != p3)	*p2 = ' ';	// restore record
			sprintf(log_str,"record = '%s'\n", *ptr);
			logit(log_str);
			sprintf(log_str,"	Invalid protocol specified, record skipped - '%s'\n",p1);
			logit(log_str);
			result = FALSE;
			goto	next;
		}
		if (protocol != PROTOCOL)	goto	next;
		if (p2 != p3)	*p2 = ' ';	// restore "original" record

		// check if additional parameter specified
		p1 = p3;
		while (WHITE(*p1))	p1++;
		p2 = p1;
		while (!WHITE(*p2) && (*p2 != '\0'))	p2++;
		if (*p2 == '\0')	{
			p3 = p2;
		}	else	{
			*p2 = '\0';
			p3 = p2 + 1;
		}
		if (*p1 != '\0')
			// don't restore record, parameters must be strings
			while (TRUE)	{
				p4 = p1;
				// multiple parameters are '%' separated
				while ((*p1 != '\0') && (*p1 != '%'))
					p1++;
				*p1 = '\0';
				if (*p4 != '\0')	{
					// add to paramter list
					slave_argv[index++] = p4;
					slave_argv[index] = 0;
				}
				p1++;
				if (p1 >= p2)	break;
			}

		// open corresponding 'control' file and spawn the
		// slave process
		if ((fd_c = open(ptr_c,O_RDONLY)) >= 0) {
			close(fd_c);
			num_h++;
			// spawn a slave process to
			// receive EDI interaction
			spawn(SLAVE_PROCESS,&slave_argv[0]);
		}	else	{
			sprintf(log_str,"Control file does not exists, record skipped -\n	'%s'\n",ptr_c);
			logit(log_str);
		}
next:	slave_argv[nparms] = 0;
	index = nparms;
	}
	if (num_h != 0)	{
		//wait for children to die
		alarm(CHILD_WAIT_TIME);
		if (sigalrm == FALSE)	sigpause(0);
	}
	sprintf(log_str,"Finished - %d host(s) processed\n",num_h);
	logit(log_str);
	release_log(result);
}


//	Spawn will 'vfork' and 'execv' a new process to process the 
//	incoming EDI transaction.  
FUNCTION spawn(char *process, char **argv)
{
	pid_t pid;
/* debug */	char	**ptr = argv;
/* debug */	sprintf(log_str,"process = '%s'\n",process);
/* debug */	logit(log_str);
/* debug */	while (*ptr)	{
/* debug */		sprintf(log_str,"argv = '%s'\n",*ptr++);
/* debug */		logit(log_str);
/* debug */	}
// if testing - return(TRUE);
	pid = vfork();
	if (pid != 0)	return(pid);
	execv(process,argv);
	perror("spawn");
	exit(1);
	return(TRUE);
}

FUNCTION void sig_handler(int sig)
{
        if (sig == SIGCHLD)     {
                wait(0);
                // restore signal handler for death of child
                sigvector((int) SIGCHLD, &vec, 0);
        }       else
                if (sig == SIGALRM)
                        // set to handle the SIGALRM signal
                        sigalrm = TRUE;
                else    {
                        sprintf(log_str,"sig_handler: Signal -%d.- ignored\n",sig);
                        logit(log_str);
                }
}
