////////////////////////////////////////////////////////////////////////
//                                                                      
//      FILE NAME : read.c                                              
//                                                                      
//      AUTHOR : John Tebbutt                                           
//                                                                      
//      DESCRIPTION : This file contains a routine to interrogate the   
//      Directory to retrieve all information for a specified trading      
//      partner on behalf of the EDI communications platform.           
//                                                                      
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

#include    <iostream.h>
#include    <fstream.h>
#include    <string.h>
#include    <malloc.h>
#include    <assert.h>
#include    <memory.h>
#include    <xom.h>
#include 	<xds.h>                // Directory Service header
extern "C"{
#include 	<xat.h>                // X/Open application tool kit header
}
#include 	<xdsbdcp.h>            // Basic Directory Contents Package header  
#include 	<xdsmdup.h>            // MHS Directory User Package header        

extern "C" {
#include	"/usr/lib/x400/gtwapi/include/nist_xmh.h"
}
#include	"/usr/lib/x400/gtwapi/include/xmhp.h"
#include	"/usr/lib/x400/gtwapi/include/ximp.h"
#include    "platform.h"
#include    "log.h"
#include    "interchange.h"
#include    "linklist.h"
#include    "or_name.h"
#include    "edi_user.h"
//
// Temporary include file name: xdsedup.h
//
#include "xdsedup.h"                // EDI Directory User Package header        
#include "jtx500.h"                 // Type, class and constant defs for X.500  
#include "rcaMacros.h"
#include "x5imports.h"				// miscellaneous declarations
#include "x5externs.h"				// miscellaneous declarations


////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
//                                                                      
//      FUNCTION NAME : read_communication_attributes()                 
//                                                                      
//      MEMBER : X500Entry                                              
//                                                                      
//      AUTHOR : John Tebbutt                                           
//                                                                      
//      PURPOSE : To read all attributes of the X.500 entry for an EDI
//      Trading Partner whose Directory name is specified by the 
//      concatenation of the two parameters with the hard-coded name of 
//      the parent entry of all EDI trading partner entries.        
//                                                                      
//      DESCRIPTION : This routine is based on the example      
//      supplied with the draft Directory Services API, IEEE 1327.2,    
//      annex B, section B.2, which reads telephone numbers                
//      of a specified target entry. The following modifications have   
//      been made in order to suit our purposes:                        
//                                                                      
//          The main routine has been renamed into a function,          
//          read_communication_attributes(). This is a member function 
//          of class X500Entry.                            
//                                                                      
//          The eDIUserName RDN of the entry's DN is assembled       
//          from two character string parameters instead of being   
//          hard coded. The same is true of the tPName RDN.
//                                                                      
//          All attributes are selected instead of just telephoneNumber.
//                                                                      
//          Instead of printing out the results, the O/R address info   
//          is mapped into an object of class X500Entry.                
//                                                                      
//          Modifications for C++ compatibility as necessary or         
//          desirable.                                                  
//                                                                      
//      RETURNED VALUES :                                               
//      The function returns a STATUS code. A value other than SUCCESS  
//      indicates an error, and the required information will not be    
//      available.                                                      
//                                                                      
//      SIDE EFFECTS : On a bad day, it can make Paul seem like a nice
//      guy... 
//                                                                      
//      BUGS, WARNINGS AND SPECIAL COMMENTS :                           
//                                                                      
//      MEMORY MANAGEMENT :                                             
//                                                                      
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

STATUS X500Entry::read_communication_attributes( char *uCode, char *uQual,
                                                    char *tPCode, char *tPQual )
{
        char                *eDIUserCN;   // common name of edi user entry
        char                *eDITPCN;     // Common name of trading
                                          // partner entry 

        DS_status           error;  	  // return value from DS functions   
		STATUS				stat;		  // return value from local functions
        OM_workspace        workspace;    // workspace for objects            
        OM_private_object   session;      // session for directory operations 
        OM_private_object   result;       // result of read operation         
        OM_sint             invoke_id;    // Invoke-ID of the read operation  
        char                *myname = "read_communication_attributes";
        int                 i;            // Loop variable 

    //
    // Build the Distinguished Name.
    // NOTE: The format of the DN is as follows: c="US",o="NIST",ou="EDI",
    // ou="users",EDIUser=uCode"-"uQual,cn=tPCode"-"tPQual.
    // By convention and for the time being, these DNs will identify aliases
    // to TP entries located under the node c="US",o="NIST",ou="EDI",ou="TPs".
    // This is to enable many users to reference multiple TPs without
    // duplicating TP information in the Directory.
    //

	// Set diagnostic print option for XAT debugging library
	print_option= PRINT_FORMAT;

	tellme( myname, "Hello\n" );

        //
        // Construct the purported common name of the EDI user entry
        //
        if( ( eDIUserCN = createName( uCode, uQual ) ) == '\0' ){

			tellme ( myname, "cannot create eDIUserCN\n" );
        }

        //
        // Construct the purported common name of the trading partner entry
        //
        if( ( eDITPCN = createName( tPCode, tPQual ) ) == '\0' ){

			tellme ( myname, "cannot create eDITPCN\n" );
        }
		tellme( myname, "Country object\n" );

// country object
		static OM_descriptor country[4];

		country[0].type = OM_CLASS;
		country[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
		country[0].value.string.length = OMP_LENGTH( DS_C_AVA );
		country[0].value.string.elements = OMP_D_DS_C_AVA;

		country[1].type = DS_ATTRIBUTE_TYPE;
		country[1].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
		country[1].value.string.length = OMP_LENGTH( DS_A_COUNTRY_NAME );
		country[1].value.string.elements = OMP_D_DS_A_COUNTRY_NAME;

		country[2].type = DS_ATTRIBUTE_VALUES;
		country[2].syntax = OM_S_PRINTABLE_STRING;
		country[2].value.string.length = sizeof( "US" );
		country[2].value.string.elements = "US";

		country[3].type = OM_NO_MORE_TYPES;
//		country[3].syntax = 0;
//		country[3].value.string.length = 0;
//		country[3].value.string.elements = 0;

/*
		tellme( myname, "Org object\n" );

// organization object
        static OM_descriptor organization[4];

        organization[0].type = OM_CLASS;
        organization[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        organization[0].value.string.length = OMP_LENGTH( DS_C_AVA );
        organization[0].value.string.elements = OMP_D_DS_C_AVA;
 
        organization[1].type = DS_ATTRIBUTE_TYPE;
        organization[1].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        organization[1].value.string.length = OMP_LENGTH( DS_A_ORGANIZATION_NAME );
        organization[1].value.string.elements = OMP_D_DS_A_ORGANIZATION_NAME;
 
        organization[2].type = DS_ATTRIBUTE_VALUES;
        organization[2].syntax = OM_S_TELETEX_STRING;
        organization[2].value.string.length = sizeof( "NIST" );
        organization[2].value.string.elements = "NIST";
 
        organization[3].type = OM_NO_MORE_TYPES;
//      organization[3].syntax = 0;
//      organization[3].value.string.length = 0;
//      organization[3].value.string.elements = 0;

		tellme( myname, "Org unit object\n" );
 
// organizational_unit object
        static OM_descriptor organizational_unit[4];
 
        organizational_unit[0].type = OM_CLASS;
        organizational_unit[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        organizational_unit[0].value.string.length = OMP_LENGTH( DS_C_AVA );
        organizational_unit[0].value.string.elements = OMP_D_DS_C_AVA;
 
        organizational_unit[1].type = DS_ATTRIBUTE_TYPE;
        organizational_unit[1].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        organizational_unit[1].value.string.length = OMP_LENGTH( DS_A_ORGANIZATIONAL_UNIT_NAME );
        organizational_unit[1].value.string.elements = OMP_D_DS_A_ORGANIZATIONAL_UNIT_NAME;
 
        organizational_unit[2].type = DS_ATTRIBUTE_VALUES;
        organizational_unit[2].syntax = OM_S_TELETEX_STRING;
        organizational_unit[2].value.string.length = sizeof( "EDI" );
        organizational_unit[2].value.string.elements = "EDI";
 
        organizational_unit[3].type = OM_NO_MORE_TYPES;
//      organizational_unit[3].syntax = 0;
//      organizational_unit[3].value.string.length = 0;
//      organizational_unit[3].value.string.elements = 0;

// eDIUserName object
//
// Temporary constant name, DS_A_EDI_NAME, 3/29/93
//
// edi-name ATTRIBUTE
//      WITH ATTRIBUTE SYNTAX caseExactStringSyntax
//      SINGLE VALUE
//      ::= id-dat-edi-routing-address  -- ::= {id-dat 1}
//                                         ::= {id-dir 1 1}
//                                         ::= {id-edims 15 1 1}
//                                         ::= {joint-iso-ccitt mhs-motis(6)
//                                                  edims(7) 15 1 1}
//

		tellme( myname, "EDI user name\n" );

        static OM_descriptor eDIUserName[4];
 
        eDIUserName[0].type = OM_CLASS;
        eDIUserName[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        eDIUserName[0].value.string.length = OMP_LENGTH( DS_C_AVA );
        eDIUserName[0].value.string.elements = OMP_D_DS_C_AVA;
 
        eDIUserName[1].type = DS_ATTRIBUTE_TYPE;
        eDIUserName[1].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        eDIUserName[1].value.string.length = OMP_LENGTH( DS_A_ORGANIZATIONAL_UNIT_NAME );
        eDIUserName[1].value.string.elements = OMP_D_DS_A_ORGANIZATIONAL_UNIT_NAME;

//      eDIUserName[1].value.string.length = OMP_LENGTH( DS_A_EDI_NAME );
//      eDIUserName[1].value.string.elements = OMP_D_DS_A_EDI_NAME;
 
        eDIUserName[2].type = DS_ATTRIBUTE_VALUES;
        eDIUserName[2].syntax = OM_S_TELETEX_STRING;
        eDIUserName[2].value.string.length = sizeof( eDIUserCN );
        eDIUserName[2].value.string.elements = eDIUserCN;
 
        eDIUserName[3].type = OM_NO_MORE_TYPES;
//      eDIUserName[3].syntax = 0;
//      eDIUserName[3].value.string.length = 0;
//      eDIUserName[3].value.string.elements = 0;
 
		tellme( myname, "Trading partner name\n" );

// tPName object
        static OM_descriptor tPName[4];

        tPName[0].type = OM_CLASS;
        tPName[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        tPName[0].value.string.length = OMP_LENGTH( DS_C_AVA );
        tPName[0].value.string.elements = OMP_D_DS_C_AVA;

        tPName[1].type = DS_ATTRIBUTE_TYPE;
        tPName[1].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        tPName[1].value.string.length = OMP_LENGTH( DS_A_COMMON_NAME );
        tPName[1].value.string.elements = OMP_D_DS_A_COMMON_NAME;
 
        tPName[2].type = DS_ATTRIBUTE_VALUES;
        tPName[2].syntax = OM_S_TELETEX_STRING;
        tPName[2].value.string.length = sizeof( eDITPCN );
        tPName[2].value.string.elements = eDITPCN;
 
        tPName[3].type = OM_NO_MORE_TYPES;
//      tPName[3].syntax = 0;
//      tPName[3].value.string.length = 0;
//      tPName[3].value.string.elements = 0;

		tellme( myname, "rdn1\n" );
*/

// rdn1 object
        static OM_descriptor rdn1[3];

        rdn1[0].type = OM_CLASS;
        rdn1[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        rdn1[0].value.string.length = OMP_LENGTH( DS_C_DS_RDN );
        rdn1[0].value.string.elements = OMP_D_DS_C_DS_RDN;

        rdn1[1].type = DS_AVAS;
        rdn1[1].syntax = OM_S_OBJECT;
        rdn1[1].value.object.padding = 0;
        rdn1[1].value.object.object = (OM_object)country;
 
        rdn1[2].type = OM_NO_MORE_TYPES;
//      rdn1[2].syntax = 0;
//      rdn1[2].value.object.length = 0;
//      rdn1[2].value.object.object = 0;

/*
		tellme( myname, "rdn2\n" );

// rdn2 object
        static OM_descriptor rdn2[3];

        rdn2[0].type = OM_CLASS;
        rdn2[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        rdn2[0].value.string.length = OMP_LENGTH( DS_C_DS_RDN );
        rdn2[0].value.string.elements = OMP_D_DS_C_DS_RDN;

        rdn2[1].type = DS_AVAS;
        rdn2[1].syntax = OM_S_OBJECT;
        rdn2[1].value.object.padding = 0;
        rdn2[1].value.object.object = (OM_object)organization;

        rdn2[2].type = OM_NO_MORE_TYPES;
//      rdn2[2].syntax = 0;
//      rdn2[2].value.string.length = 0;
//      rdn2[2].value.string.elements = 0;

		tellme( myname, "rdn3\n" );

// rdn3 object
        static OM_descriptor rdn3[3];

        rdn3[0].type = OM_CLASS;
        rdn3[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        rdn3[0].value.string.length = OMP_LENGTH( DS_C_DS_RDN );
        rdn3[0].value.string.elements = OMP_D_DS_C_DS_RDN;

        rdn3[1].type = DS_AVAS;
        rdn3[1].syntax = OM_S_OBJECT;
        rdn3[1].value.object.padding = 0;
        rdn3[1].value.object.object = (OM_object)organizational_unit;

        rdn3[2].type = OM_NO_MORE_TYPES;
//      rdn3[2].syntax = 0;
//      rdn3[2].value.string.length = 0;
//      rdn3[2].value.string.elements = 0;

		tellme( myname, "rdn4\n" );

// rdn4 object
        static OM_descriptor rdn4[3];

        rdn4[0].type = OM_CLASS;
        rdn4[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        rdn4[0].value.string.length = OMP_LENGTH( DS_C_DS_RDN );
        rdn4[0].value.string.elements = OMP_D_DS_C_DS_RDN;
 
        rdn4[1].type = DS_AVAS;
        rdn4[1].syntax = OM_S_OBJECT;
        rdn4[1].value.object.padding = 0;
        rdn4[1].value.object.object = (OM_object)eDIUserName;

        rdn4[2].type = OM_NO_MORE_TYPES;
//      rdn4[2].syntax = 0;
//      rdn4[2].value.string.length = 0;
//      rdn4[2].value.string.elements = 0;

		tellme( myname, "rdn5\n" );

// rdn5 object
        static OM_descriptor rdn5[3];

        rdn5[0].type = OM_CLASS;
        rdn5[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        rdn5[0].value.string.length = OMP_LENGTH( DS_C_DS_RDN );
        rdn5[0].value.string.elements = OMP_D_DS_C_DS_RDN;
 
        rdn5[1].type = DS_AVAS;
        rdn5[1].syntax = OM_S_OBJECT;
        rdn5[1].value.object.padding = 0;
        rdn5[1].value.object.object = (OM_object)tPName;

        rdn5[2].type = OM_NO_MORE_TYPES;
//      rdn5[2].syntax = 0;
//      rdn5[2].value.string.length = 0;
//      rdn5[2].value.string.elements = 0;

*/
		tellme( myname, "name\n" );

// name object
        OM_descriptor name[3];

        name[0].type = OM_CLASS;
        name[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        name[0].value.string.length = OMP_LENGTH( DS_C_DS_DN );
        name[0].value.string.elements = OMP_D_DS_C_DS_DN;
 
        name[1].type = DS_RDNS;
        name[1].syntax = OM_S_OBJECT;
        name[1].value.object.padding = 0;
        name[1].value.object.object = (OM_object)rdn1;

/*
        name[2].type = DS_RDNS;
        name[2].syntax = OM_S_OBJECT;
        name[2].value.object.padding = 0;
        name[2].value.object.object = (OM_object)rdn2;

        name[3].type = DS_RDNS;
        name[3].syntax = OM_S_OBJECT;
        name[3].value.object.padding = 0;
        name[3].value.object.object = (OM_object)rdn3;

        name[4].type = DS_RDNS;
        name[4].syntax = OM_S_OBJECT;
        name[4].value.object.padding = 0;
        name[4].value.object.object = (OM_object)rdn4;

        name[5].type = DS_RDNS;
        name[5].syntax = OM_S_OBJECT;
        name[5].value.object.padding = 0;
        name[5].value.object.object = (OM_object)rdn5;

		*/
        name[6].type = OM_NO_MORE_TYPES;
//      name[6].syntax = 0;
//      name[6].value.string.length = 0;
//      name[6].value.string.elements = 0;

/*
    //
    // Public Object ("Descriptor List") for Entry-Information-Selection
    // argument to ds_read().
    //
    // We require all values of all attributes...
    //

		tellme( myname, "Entry information selection\n" );

// selection object
        static OM_descriptor selection[4];

        //
        // Class attribute (This object represents entryInformation-
        // Selection information)
        //
        selection[0].type = OM_CLASS;
        selection[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        selection[0].value.string.length = 
								OMP_LENGTH( DS_C_ENTRY_INFORMATION_SELECTION );
        selection[0].value.string.elements = 
										OMP_D_DS_C_ENTRY_INFORMATION_SELECTION;
 
        //
        // Attributes-Selected (All attributes)
        //
        selection[1].type = DS_ALL_ATTRIBUTES;
        selection[1].syntax = OM_S_BOOLEAN;
        selection[1].value.boolean = OM_TRUE;

        //
        // Information-Type (Attribute types and values)
        //
		selection[2].type = DS_INFORMATION_TYPE;                            
        selection[2].syntax = OM_S_ENUMERATION;  
        selection[2].value.enumeration = DS_TYPES_AND_VALUES;

        //
        // Object terminator (That's all, folks!)
        //
        selection[3].type = OM_NO_MORE_TYPES;
		*/

    // 
    // variables to extract information from the read result...
    //
	// This is ridiculous. HP C++ compiler would not allow the following
	// initialization statements:
    // OM_type     entry_list[]        = { DS_ENTRY, 0 };
    // OM_type     attributes_list[]   = { DS_ATTRIBUTES, 0 };
    // OM_type     values_list[]       = { DS_ATTRIBUTE_VALUES, 0 };
	//
	// ...so we have to type it all out longhand...
	//
	OM_type     entry_list[2];
	OM_type     attributes_list[2];
	OM_type     values_list[2];
	OM_type     types_list[2];

	entry_list[0] = DS_ENTRY;
	entry_list[1] = 0;

	attributes_list[0] = DS_ATTRIBUTES;
	attributes_list[1] = 0;

	values_list[0] = DS_ATTRIBUTE_VALUES;
	values_list[1] = 0;

	types_list[0] = DS_ATTRIBUTE_TYPE;
	types_list[1] = 0;


    OM_public_object    entry;
    OM_public_object    attributes;
    OM_public_object    thisAttr;     // current attribute 
    OM_value_position   total_number, // see om_get() info below 
						level_2_total;
    OM_object_identifier entryOC; 	  // the object class of the directory entry

    //
    // Perform the Directory operations:
    // (1) Initialise the Directory Service and get an OM workspace.
    // (2) Bind a default directory session.
    // (3) Read the O/R addresses of "name".
    // (4) Terminate the directory session.
    //
    // Note: The ds_initialize() and ds_shutdown() calls were listed
    //       incorrectly in the XDS specification- ds_initialize() returns
    //       a workspace handle as its return value, not as an argument-
    //       and returns NULL is unsuccessful- hence the CHECK_DS_CALL()
    //       call was changed to report an error if workspace was equal to
    //       0 (NULL).  ds_shutdown() returns no values and takes no 
    //       values, hence making a CHECK_DS_CALL() didn't make much sense.
    //
     
    
	tellme( myname, "Calling ds_initialize()\n" );

    if( ( ( workspace = (OM_workspace)ds_initialize() ) == DS_NULL_RESULT ) ){

		cerr << myname << "Workspace initialization failure";
		goto cleanUpAndExit;
	}

	tellme( myname, "Making tiny session object\n" );

	OM_object tiny_session, pAddr;

	// Here is our session object...
	tiny_session = at_make_object( DS_C_SESSION, 4 );

	// First, we create the presentation addr of the called DSA...
	pAddr = at_make_object( DS_C_PRESENTATION_ADDRESS, 4 );

	pAddr[1].type = DS_N_ADDRESSES;
	pAddr[1].syntax = OM_S_OCTET_STRING;
	pAddr[1].value.string.length = sizeof( "0x47000580005A0000000001e3708000930142001" );
	pAddr[1].value.string.elements = "0x47000580005A0000000001e3708000930142001";

	pAddr[2].type = DS_P_SELECTOR;
	pAddr[2].syntax = OM_S_OCTET_STRING;
	pAddr[2].value.string.length = sizeof( "dsa" );
	pAddr[2].value.string.elements = "dsa";

	pAddr[3].type = DS_S_SELECTOR;
	pAddr[3].syntax = OM_S_OCTET_STRING;
	pAddr[3].value.string.length = sizeof( "63" );
	pAddr[3].value.string.elements = "63";

	pAddr[4].type = DS_T_SELECTOR;
	pAddr[4].syntax = OM_S_OCTET_STRING;
	pAddr[4].value.string.length = sizeof( "62" );
	pAddr[4].value.string.elements = "62";

	// Now fill in the session object...
	tiny_session[1].type = DS_DSA_ADDRESS;
	tiny_session[1].syntax = OM_S_OBJECT;
	tiny_session[1].value.object.padding = 0;
	tiny_session[1].value.object.object = pAddr;

	tiny_session[2].type = DS_DSA_NAME;
	tiny_session[2].syntax = OM_S_OBJECT;
	tiny_session[2].value.object.padding = 0;
	tiny_session[2].value.object.object = 
								at_string_to_dn( "/C=US/O=NIST/CN=DSA1" );

	tiny_session[3].type = DS_FILE_DESCRIPTOR;
	tiny_session[3].syntax = OM_S_INTEGER;
	tiny_session[3].value.integer = 0;

	tiny_session[4].type = DS_REQUESTOR;
	tiny_session[4].syntax = OM_S_OBJECT;
	tiny_session[4].value.object.padding = 0;
	tiny_session[4].value.object.object = 
								at_string_to_dn( "/C=US/" );
	

	tellme( myname, "Session parameter to ds_bind()\n" );

	at_print_object( (OM_object)tiny_session );

	tellme( myname, "Calling ds_bind()\n" );

    CHECK_DS_CALL( ds_bind( DS_DEFAULT_SESSION, &session ) );

//    CHECK_DS_CALL( ds_bind( tiny_session, &session ) );

	tellme( myname, "Let's have a look at the session returned from ds_bind\n");

	at_print_object( (OM_object)session );

	tellme( myname, "Generating short name using at_string_to_dn()\n" );

	OM_object newname; 

	newname = at_string_to_dn("C=US");

	tellme( myname, "DN of object follows\n");

	at_print_object( (OM_object)newname );

	tellme( myname, "Calling ds_read()\n" );

//  if( ( error = (DS_status)ds_read(	session, 
    if( ( error = (DS_status)ds_read(	tiny_session, 
                            (OM_private_object)DS_DEFAULT_CONTEXT, 
//                            (OM_object)name, 
							newname,
							DS_SELECT_ALL_TYPES_AND_VALUES,
//                            (OM_object)selection,
                            &result, 
                            &invoke_id ) ) 
							
			!= DS_SUCCESS ){
		
		tellme( myname, "ds_read() failure. Error follows \n" );

		at_print_object( (OM_object)error );

		tellme( myname, "Calling ds_unbind()\n" );

    	CHECK_DS_CALL( ds_unbind( session ) );

		goto cleanUpAndExit;

	}

	tellme( myname, "Calling ds_unbind()\n" );

    CHECK_DS_CALL( ds_unbind( session ) );

    //
    // Extract the attribute values of "name" from the result
    //
    // There are 4 stages:
    // (1) get the Entry-Information from the Read-Result.
    // (2) get the Attributes from the Entry-Information.
    // (3) get the list of values for each attribute
    // (4) map the info into our local X500Entry.
    //
    // Note: The om_get() calls were listed incorrectly in the XDS 
    //       specification- om_get() returns an additional parameter,
    //       named total_number, which is an OM_value_position/.  Its
    //       value will be set by om_get() to the number of attribute 
    //       descriptors returned in the public entry.
    //
     
    //
    // Get reference to entry info from result...
    //

	tellme( myname, "Get entry info\n" );

    CHECK_OM_CALL(  om_get(result,
                OM_EXCLUDE_ALL_BUT_CERTAIN_TYPES
            +   OM_EXCLUDE_SUBOBJECTS,
                entry_list, OM_FALSE, 0, 0, &entry, 
                &total_number) );


    //
    // Get reference to the attributes from the entry...
    // (We have to use entry->value.object.object because the OM_OBJ_VALUE
	// macro is not supplied by the API implementor)

	tellme( myname, "Get attributes\n" );

    CHECK_OM_CALL(  om_get( entry->value.object.object,
                OM_EXCLUDE_ALL_BUT_CERTAIN_TYPES + OM_EXCLUDE_SUBOBJECTS,
                attributes_list, OM_FALSE, 0, 0, &attributes,
                &total_number) );

//
// attributes now holds an array of struct OM_descriptor_struct, each element
// of which represents an X.500 attribute from the target entry...
//
// Obtain the object class of the entry by stepping through the list of 
// attributes that make up the entry until we find the object class attribute.
//
for( i = 0; i < total_number; ++i ){

	//
	// Get the OM object which holds the DS attribute type of the DS attribute
	// represented by attributes[i]
	//

	tellme( myname, form( "Searching for attribute type: %n\n", i ) );

    CHECK_OM_CALL(  om_get(attributes[i].value.object.object,
                OM_EXCLUDE_ALL_BUT_CERTAIN_TYPES
            +   OM_EXCLUDE_SUBOBJECTS,
                types_list, OM_FALSE, 0, 0, &thisAttr,
                &level_2_total) );
 
	//
	// ...now check to see if this is the attribute which holds the entry's
	// object class...
	//

	tellme( myname, "Checking for object class attribute\n" );

	if( memcmp( (char *)thisAttr->value.string.elements, 
				(char *)OMP_O_DS_A_OBJECT_CLASS,
				(int)thisAttr->value.string.length ) == 0 ) {
 
            entryOC = thisAttr->value.string;
	}

}   // End for( i = 0;...

if( entryOC.length == 0 ){      // We didn't find the entry's object class...

    tellme( myname, "Cannot find entry object class\n" );
    stat = UNSUCCESSFUL;
    goto cleanUpAndExit;
}

//
// Determine the value of the entry's object class...
//
// MHS-EDI
//
if( memcmp( (char *)entryOC.elements, 
            (char *)OMP_O_DS_O_EDI_TP_MHS_EDI,
            (int)entryOC.length ) == 0 ){

	tellme( myname, "Entry is class MHS-EDI\n" );

    stat = mapMhsEdiAttrs( attributes, total_number );
}
//
// MHS-IPM
//
else if( memcmp((char *)entryOC.elements, 
                (char *)OMP_O_DS_O_EDI_TP_MHS_IPM,
                (int)entryOC.length ) == 0 ){

	tellme( myname, "Entry is class MHS-IPM\n" );

    stat = mapMhsIpmAttrs( attributes, total_number );
 }
//
// VAN
//
else if( memcmp((char *)entryOC.elements, 
                (char *)OMP_O_DS_O_EDI_TP_VAN,
                (int)entryOC.length ) == 0 ){

	tellme( myname, "Entry is class VAN\n" );

    stat = mapVanAttrs( attributes, total_number );
 }
//
// FTAM
//
else if( memcmp((char *)entryOC.elements, 
                (char *)OMP_O_DS_O_EDI_TP_FTAM,
                (int)entryOC.length ) == 0 ){

	tellme( myname, "Entry is class FTAM\n" );

    stat = mapFtamAttrs( attributes, total_number );
}
//
// UNKNOWN...
//
else {

    stat = UNSUCCESSFUL;
    tellme( myname, form( "Unknown Object Class: %20s", (char *)entryOC.elements) );

//  clean up...
}

if( stat != SUCCESS ){

    tellme( myname, "Unable to map entry attributes into local struct\n" );
    goto cleanUpAndExit;
}

cleanUpAndExit:

    //
    //  We can now safely release all the private objects
    //  and the public objects we no longer need
    //

	tellme( "cleanUpAndExit", "WE ARE OUTTA HERE!!!\n" );
     
    CHECK_OM_CALL(om_delete(session)) ;
    CHECK_OM_CALL(om_delete(result)) ;
    CHECK_OM_CALL(om_delete(entry)) ;
    CHECK_OM_CALL(om_delete(attributes)) ;

	tellme( myname, "Calling ds-shutdown()\n" );

    ds_shutdown() ;

	tellme(myname, "Over and out\n" );

    return( stat );
}


//
// the error handling code
//
// NOTE: any errors arising in these functions are ignored.
// NOTE: it is assumed that om_delete() is sufficiently intelligent
// not to try to access any memory pointed to by error without first 
// checking that the value in error is not one of the various object
// constants defined in IEEE 1327.2/D4-1992, section 3.3.3.
//

void handle_om_error(OM_return_code return_code)
{
char * myname = "handle_om_error";

	tellme( myname, "called\n" );
    cerr << "OM error " << return_code << "has occurred\n";

//
//  Keep it (too) simple for the time being...
//

    return;
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
//
//      FUNCTION NAME : mapMhsEdiAttrs()
//
//      MEMBER : X500Entry
//
//      AUTHOR : John Tebbutt
//
//      PURPOSE : To map attribute values from an array of public objects
//      into the local structure ("this") in the case where these attributes
//      represent an MHS EDI trading partner entry.
//
//      DESCRIPTION : Set the protocol_id field. Step through the array of 
//      objects. For each object, check the attribute type of the attribute 
//      it represents. If the attribute is of a known type, map appropriately, 
//      else report unknown attribute type.
//
//      RETURNED VALUES :
//      The function returns a STATUS code. A value other than SUCCESS
//      indicates an error, and the required information will not be
//      available.
//
//      SIDE EFFECTS : It looks good, it tastes good and, by golly, it
//      does you good! 
//
//      BUGS, WARNINGS AND SPECIAL COMMENTS :
//
//      MEMORY MANAGEMENT :
//
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
 
STATUS X500Entry::mapMhsEdiAttrs(   OM_public_object    attributes, 
                                    OM_value_position   total )
{

int         		i;
OM_public_object	attrType, attrValue;
OM_value_position   typeTotal, valueTotal;
OM_type     		attributes_list[2];
OM_type     		values_list[2];
OM_type     		types_list[2];
OM_type             oR_addrs_list[2];
char				*myname = "mapMhsEdiAttrs";

attributes_list[0] = DS_ATTRIBUTES;
attributes_list[1] = 0;

values_list[0] = DS_ATTRIBUTE_VALUES;
values_list[1] = 0;

types_list[0] = DS_ATTRIBUTE_TYPE;
types_list[1] = 0;

tellme(myname, "Hey, there, TC!\n" );

//
// attributes holds an array of struct OM_descriptor_struct, each element
// of which represents an X.500 attribute from the target entry...
//
// Step through the array of attributes, testing each one for its type and 
// mapping the values appropriately...
//

tellme( myname, "Stepping through attribute array...\n" );

for( i = 0; i < total; ++i ){

	//
	// Get the OM object which holds the DS attribute type of the DS attribute
	// represented by attributes[i]
	//

	tellme( myname, "Get attribute type\n" );

    CHECK_OM_CALL(  om_get(attributes[i].value.object.object,
                OM_EXCLUDE_ALL_BUT_CERTAIN_TYPES
            +   OM_EXCLUDE_SUBOBJECTS,
                types_list, OM_FALSE, 0, 0, &attrType,
                &typeTotal) );

	//
	// ...and the attribute value associated with the type...
	//

	tellme( myname, "Get attribute value\n" );

    CHECK_OM_CALL(  om_get(attributes[i].value.object.object,
                OM_EXCLUDE_ALL_BUT_CERTAIN_TYPES
            +   OM_EXCLUDE_SUBOBJECTS,
                values_list, OM_FALSE, 0, 0, &attrValue,
                &valueTotal) );
 
	//
	// ...and compare the attribute type against those expected for
	// the entry...

	tellme( myname, "Compare attributes against those expected for the entry\n");
	//
	// this-recipient
	//

	if( memcmp( (char *)attrType->value.string.elements, 
				(char *)OMP_O_DS_A_EDI_TR,
				(int)attrType->value.string.length ) == 0 ) {
 
		tellme( myname, "Found this-recipient\n" );

		this_recip = (ORName *)attrValue->value.object.object;
		
	}

	//
	// mhs-origination-security
	//

	else if( memcmp( (char *)attrType->value.string.elements,
                (char *)OMP_O_DS_A_EDI_MHS_ORIG_SEC,
                (int)attrType->value.string.length ) == 0 ) {

		tellme( myname, "Found mhs-origination-security\n" );

        orig_security = (int)attrValue->value.integer;

    }

	//
	// priority
	//

	else if( memcmp( (char *)attrType->value.string.elements,
                (char *)OMP_O_DS_A_EDI_PRIORITY,
                (int)attrType->value.string.length ) == 0 ) {

		tellme( myname, "Found priority\n" );

        prior = (int)attrValue->value.integer;

    }

	//
	// eNRFTR-pn
	//

    else if( memcmp( (char *)attrType->value.string.elements,
                (char *)OMP_O_DS_A_EDI_ENRFTR_PN,
                (int)attrType->value.string.length ) == 0 ) {
 
		tellme( myname, "Found eNRFTR-pn\n" );

        positive_notif = (BOOL)attrValue->value.boolean;
 
    }
 
	//
	// eNRFTR-nn
	//

    else if( memcmp( (char *)attrType->value.string.elements, 
                (char *)OMP_O_DS_A_EDI_ENRFTR_NN, 
                (int)attrType->value.string.length ) == 0 ) { 
  
		tellme( myname, "Found eNRFTR-nn\n" );

        negative_notif = (BOOL)attrValue->value.boolean; 
  
    } 

	//
	// eNRFTR-fn
	//

    else if( memcmp( (char *)attrType->value.string.elements, 
                (char *)OMP_O_DS_A_EDI_ENRFTR_FN, 
                (int)attrType->value.string.length ) == 0 ) { 
  
		tellme( myname, " Found eNRFTR-fn\n" );

        forwarded_notif = (BOOL)attrValue->value.boolean; 
  
    } 
 
	//
	// eNSFTR-proof
	//

    else if( memcmp( (char *)attrType->value.string.elements, 
                (char *)OMP_O_DS_A_EDI_ENSFTR_PR, 
                (int)attrType->value.string.length ) == 0 ) { 
  
		tellme( myname, "Found eNSFTR-proof\n" );

        proof_of_notif = (BOOL)attrValue->value.boolean; 
  
    }

	//
	// eNSFTR-non-repudiation
	//

    else if( memcmp( (char *)attrType->value.string.elements, 
                (char *)OMP_O_DS_A_EDI_ENSFTR_N_R, 
                (int)attrType->value.string.length ) == 0 ) { 
  
		tellme( myname, "Found eNSFTR-non-repudiation\n" );

        nonrepud_of_notif = (BOOL)attrValue->value.boolean; 
  
    } 
 
	//
	// eRSFTR-proof
	//

    else if( memcmp( (char *)attrType->value.string.elements, 
                (char *)OMP_O_DS_A_EDI_ERSFTR_PR, 
                (int)attrType->value.string.length ) == 0 ) { 
  
		tellme( myname, "Found eRSFTR-proof\n" );

        proof_of_rcpt = (BOOL)attrValue->value.boolean; 
  
    }
 
	//
	// eRSFTR-non-repudiation
	//

    else if( memcmp( (char *)attrType->value.string.elements,  
                (char *)OMP_O_DS_A_EDI_ERSFTR_N_R,  
                (int)attrType->value.string.length ) == 0 ) {  
  
		tellme( myname, "Found eRSFTR-non-repudiation\n" );

        nonrepud_of_rcpt = (BOOL)attrValue->value.boolean;  
   
    }  
 
	//
	// edin-receiver-name
	//

	else if( memcmp( (char *)attrType->value.string.elements,
                (char *)OMP_O_DS_A_EDI_EDIN_REC_NAME,
                (int)attrType->value.string.length ) == 0 ) {
 
		tellme( myname, "Found edin-receiver-name\n" );

        edin_receiver = (ORName *)attrValue->value.object.object; 
         
    }

	//
	// responsibility-passing-allowed-for-this-recipient
	//

    else if( memcmp( (char *)attrType->value.string.elements,  
                (char *)OMP_O_DS_A_EDI_RPA_FTR,  
                (int)attrType->value.string.length ) == 0 ) {  
  
		tellme( myname, "Found responsibility-passing-allowed-for-this-recipient\n" );

        resp_passing = (BOOL)attrValue->value.boolean;  
   
    }

	else{

		tellme( myname, form( "Unrecognized attribute type: %20s",
							hex( *(int *)attrType->value.string.elements) ) );
	}
 
}   // End for( i = 0;...

tellme( myname, " Goodbye\n" );

return( SUCCESS );

}   // End of mapMhsEdiAttrs()


////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
//
//      FUNCTION NAME : mapMhsIpmAttrs()
//
//      MEMBER : X500Entry
//
//      AUTHOR : John Tebbutt
//
//      PURPOSE : To map attribute values from an array of public objects
//      into the local structure ("this") in the case where these attributes
//      represent an MHS IPM trading partner entry.
//
//      DESCRIPTION : Set the protocol_id field. Step through the array of 
//      objects. For each object, check the attribute type of the attribute 
//      it represents. If the attribute is of a known type, map appropriately, 
//      else report unknown attribute type.
//
//      RETURNED VALUES :
//      The function returns a STATUS code. A value other than SUCCESS
//      indicates an error, and the required information will not be
//      available.
//
//      SIDE EFFECTS : It looks good, it tastes good and, by golly, it
//      does you good! 
//
//      BUGS, WARNINGS AND SPECIAL COMMENTS :
//
//      MEMORY MANAGEMENT :
//
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
 
STATUS X500Entry::mapMhsIpmAttrs(   OM_public_object    attributes, 
                                    OM_value_position   total )
{

int         		i;
OM_public_object	attrType, attrValue;
OM_value_position   typeTotal, valueTotal;
OM_type     		attributes_list[2];
OM_type     		values_list[2];
OM_type     		types_list[2];
OM_type             oR_addrs_list[2];
char				*myname = "mapMhsIpmAttrs";

attributes_list[0] = DS_ATTRIBUTES;
attributes_list[1] = 0;

values_list[0] = DS_ATTRIBUTE_VALUES;
values_list[1] = 0;

types_list[0] = DS_ATTRIBUTE_TYPE;
types_list[1] = 0;

tellme( myname, "Here he is...\n" );

//
// attributes holds an array of struct OM_descriptor_struct, each element
// of which represents an X.500 attribute from the target entry...
//
// Step through the array of attributes, testing each one for its type and 
// mapping the values appropriately...
//

tellme( myname, "Stepping through array of attributes...\n" );

for( i = 0; i < total; ++i ){

	//
	// Get the OM object which holds the DS attribute type of the DS attribute
	// represented by attributes[i]
	//
    CHECK_OM_CALL(  om_get(attributes[i].value.object.object,
                OM_EXCLUDE_ALL_BUT_CERTAIN_TYPES
            +   OM_EXCLUDE_SUBOBJECTS,
                types_list, OM_FALSE, 0, 0, &attrType,
                &typeTotal) );

	//
	// ...and the attribute value associated with the type...
	//
    CHECK_OM_CALL(  om_get(attributes[i].value.object.object,
                OM_EXCLUDE_ALL_BUT_CERTAIN_TYPES
            +   OM_EXCLUDE_SUBOBJECTS,
                values_list, OM_FALSE, 0, 0, &attrValue,
                &valueTotal) );
 
	//
	// ...and compare the attribute type against those expected for
	// the entry...
	//
	// this-recipient
	//
	if( memcmp( (char *)attrType->value.string.elements, 
				(char *)OMP_O_DS_A_EDI_TR,
				(int)attrType->value.string.length ) == 0 ) {

		tellme( myname, "Found this_recip\n" );
 
		this_recip = (ORName *)attrValue->value.object.object;
		
	}

	//
	// mhs-origination-security
	//
	else if( memcmp( (char *)attrType->value.string.elements,
                (char *)OMP_O_DS_A_EDI_MHS_ORIG_SEC,
                (int)attrType->value.string.length ) == 0 ) {

        tellme( myname, "Found orig_security\n" );

        orig_security = (int)attrValue->value.integer;

    }

	//
	// priority
	//
	else if( memcmp( (char *)attrType->value.string.elements,
                (char *)OMP_O_DS_A_EDI_PRIORITY,
                (int)attrType->value.string.length ) == 0 ) {

        tellme( myname, "Found prior\n" );

        prior = (int)attrValue->value.integer;

    }

	else{

		tellme( myname, form( "Unrecognized attribute type: %20s",
							hex( *(int *)attrType->value.string.elements) ) );
	}
 
}   // End for( i = 0;...

tellme( myname, "Goodbye\n" );

return( SUCCESS );

}   // End of mapMhsIpmAttrs()

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
//
//      FUNCTION NAME : mapFtamAttrs()
//
//      MEMBER : X500Entry
//
//      AUTHOR : John Tebbutt
//
//      PURPOSE : To map attribute values from an array of public objects
//      into the local structure ("this") in the case where these attributes
//      represent an FTAM trading partner entry.
//
//      DESCRIPTION : Set the protocol_id field. Step through the array of 
//      objects. For each object, check the attribute type of the attribute 
//      it represents. If the attribute is of a known type, map appropriately, 
//      else report unknown attribute type.
//
//      RETURNED VALUES :
//      The function returns a STATUS code. A value other than SUCCESS
//      indicates an error, and the required information will not be
//      available.
//
//      SIDE EFFECTS : It looks good, it tastes good and, by golly, it
//      does you good! 
//
//      BUGS, WARNINGS AND SPECIAL COMMENTS :
//
//      MEMORY MANAGEMENT :
//
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
 
STATUS X500Entry::mapFtamAttrs(   OM_public_object    attributes, 
                                    OM_value_position   total )
{

int         		i;
OM_public_object	attrType, attrValue;
OM_value_position   typeTotal, valueTotal;
OM_type     		attributes_list[2];
OM_type     		values_list[2];
OM_type     		types_list[2];
OM_type             oR_addrs_list[2];
char				*myname = "mapFtamAttrs";

attributes_list[0] = DS_ATTRIBUTES;
attributes_list[1] = 0;

values_list[0] = DS_ATTRIBUTE_VALUES;
values_list[1] = 0;

types_list[0] = DS_ATTRIBUTE_TYPE;
types_list[1] = 0;

//
// attributes holds an array of struct OM_descriptor_struct, each element
// of which represents an X.500 attribute from the target entry...
//
// Step through the array of attributes, testing each one for its type and 
// mapping the values appropriately...
//

tellme( myname, "Stepping through array of attributes...\n" );

for( i = 0; i < total; ++i ){

	//
	// Get the OM object which holds the DS attribute type of the DS attribute
	// represented by attributes[i]
	//
    CHECK_OM_CALL(  om_get(attributes[i].value.object.object,
                OM_EXCLUDE_ALL_BUT_CERTAIN_TYPES
            +   OM_EXCLUDE_SUBOBJECTS,
                types_list, OM_FALSE, 0, 0, &attrType,
                &typeTotal) );

	//
	// ...and the attribute value associated with the type...
	//
    CHECK_OM_CALL(  om_get(attributes[i].value.object.object,
                OM_EXCLUDE_ALL_BUT_CERTAIN_TYPES
            +   OM_EXCLUDE_SUBOBJECTS,
                values_list, OM_FALSE, 0, 0, &attrValue,
                &valueTotal) );
 
	//
	// ...and compare the attribute type against those expected for
	// the entry...
	//
	// connection-address
	//
	if( memcmp( (char *)attrType->value.string.elements, 
				(char *)OMP_O_DS_A_EDI_FTAM_CONN_ADDR,
				(int)attrType->value.string.length ) == 0 ) {

        tellme( myname, "Found name\n" );
 
		strncpy( name, (char *)attrValue->value.string.elements,
							(int)attrValue->value.string.length );
		
	}

	//
	// user-password
	//
	else if( memcmp( (char *)attrType->value.string.elements,
                (char *)OMP_O_DS_A_EDI_FTAM_PASSWORD,
                (int)attrType->value.string.length ) == 0 ) {

        tellme( myname, "Found password\n" );

		strncpy( password, (char *)attrValue->value.string.elements,
							(int)attrValue->value.string.length );

    }

	//
	// user-name
	//
	else if( memcmp( (char *)attrType->value.string.elements,
                (char *)OMP_O_DS_A_EDI_FTAM_X500_NAME,
                (int)attrType->value.string.length ) == 0 ) {

        tellme( myname, "Found x500_name\n" );

		strncpy( x500_name, (char *)attrValue->value.string.elements,
							(int)attrValue->value.string.length );

    }

	//
	// file-directory
	//
	else if( memcmp( (char *)attrType->value.string.elements,
                (char *)OMP_O_DS_A_EDI_FTAM_DIR_NAME,
                (int)attrType->value.string.length ) == 0 ) {

        tellme( myname, "Found dir_name\n" );

		strncpy( dir_name, (char *)attrValue->value.string.elements,
							(int)attrValue->value.string.length );
	}

	else{

		tellme( myname, form( "Unrecognized attribute type: %20s",
							hex( *(int *)attrType->value.string.elements ) ) ); 
	}
 
}   // End for( i = 0;...

tellme( myname, "Goodbye\n" );

return( SUCCESS );

}   // End of mapFtamAttrs()

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
//
//      FUNCTION NAME : mapVanAttrs()
//
//      MEMBER : X500Entry
//
//      AUTHOR : John Tebbutt
//
//      PURPOSE : To map attribute values from an array of public objects
//      into the local structure ("this") in the case where these attributes
//      represent an VAN trading partner entry.
//
//      DESCRIPTION : Set the protocol_id field. Step through the array of 
//      objects. For each object, check the attribute type of the attribute 
//      it represents. If the attribute is of a known type, map appropriately, 
//      else report unknown attribute type.
//
//      RETURNED VALUES :
//      The function returns a STATUS code. A value other than SUCCESS
//      indicates an error, and the required information will not be
//      available.
//
//      SIDE EFFECTS : It looks good, it tastes good and... blech.
//
//      BUGS, WARNINGS AND SPECIAL COMMENTS :
//
//      MEMORY MANAGEMENT :
//
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
 
STATUS X500Entry::mapVanAttrs(   OM_public_object    attributes, 
                                    OM_value_position   total )
{

int         		i;
OM_public_object	attrType, attrValue;
OM_value_position   typeTotal, valueTotal;
OM_type     		attributes_list[2];
OM_type     		values_list[2];
OM_type     		types_list[2];
OM_type             oR_addrs_list[2];
char				*myname = "mapVanAttrs";

attributes_list[0] = DS_ATTRIBUTES;
attributes_list[1] = 0;

values_list[0] = DS_ATTRIBUTE_VALUES;
values_list[1] = 0;

types_list[0] = DS_ATTRIBUTE_TYPE;
types_list[1] = 0;

//
// attributes holds an array of struct OM_descriptor_struct, each element
// of which represents an X.500 attribute from the target entry...
//
// Step through the array of attributes, testing each one for its type and 
// mapping the values appropriately...
//

tellme( myname, "Stepping through array of attributes...\n" );

for( i = 0; i < total; ++i ){

	//
	// Get the OM object which holds the DS attribute type of the DS attribute
	// represented by attributes[i]
	//
    CHECK_OM_CALL(  om_get(attributes[i].value.object.object,
                OM_EXCLUDE_ALL_BUT_CERTAIN_TYPES
            +   OM_EXCLUDE_SUBOBJECTS,
                types_list, OM_FALSE, 0, 0, &attrType,
                &typeTotal) );

	//
	// ...and the attribute value associated with the type...
	//
    CHECK_OM_CALL(  om_get(attributes[i].value.object.object,
                OM_EXCLUDE_ALL_BUT_CERTAIN_TYPES
            +   OM_EXCLUDE_SUBOBJECTS,
                values_list, OM_FALSE, 0, 0, &attrValue,
                &valueTotal) );
 
	//
	// ...and compare the attribute type against those expected for
	// the entry...
	//
	// VAN name
	//
	if( memcmp( (char *)attrType->value.string.elements, 
				(char *)OMP_O_DS_A_EDI_VAN_NAME,
				(int)attrType->value.string.length ) == 0 ) {

        tellme( myname, "Found vanname\n" );
 
		strncpy( vanname, (char *)attrValue->value.string.elements,
							(int)attrValue->value.string.length );
		
	}

	else{

		tellme( myname, form( "Unrecognized attribute type: %20s",
							hex( *(int *)attrType->value.string.elements ) )
);
	}
 
}   // End for( i = 0;...

tellme( myname, "Goodbye\n" );

return( SUCCESS );

}   // End of mapVanAttrs()
