////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
//                                                                      
//      FILE NAME : validate.c                                              
//                                                                      
//      AUTHOR : John Tebbutt                                           
//                                                                      
//      DESCRIPTION : This file contains routines to validate purported
//      trading partners against trading partner relationship information
//      stored in the Directory. All this in the name of NIST's EDI
//      communications platform.
//                                                                      
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

#include    <iostream.h>
#include    <fstream.h>
#include    <string.h>
#include    <assert.h>
#include    <xom.h>
#include    <xds.h>                // Directory Service header
extern "C"{
#include 	<xat.h>                // X/Open application tool kit header
}
#include    <xdsbdcp.h>            // Basic Directory Contents Package header  
#include    <xdsmdup.h>            // MHS Directory User Package header        
//
// Temporary include file name: xdsedup.h
//
extern "C" {
#include	"/usr/lib/x400/gtwapi/include/nist_xmh.h"
}
#include	"/usr/lib/x400/gtwapi/include/xmhp.h"
#include	"/usr/lib/x400/gtwapi/include/ximp.h"
#include    "xdsedup.h"            // EDI Directory User Package header        
#include    "platform.h"
#include    "log.h"
#include    "interchange.h"
#include    "linklist.h"
#include    "or_name.h"
#include    "edi_user.h"
#include 	"jtx500.h"              // Type, class and constant defs for X.500  
#include 	"x5imports.h"
#include 	"x5externs.h"

DS_status           		error;        // return value from DS functions

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
//                                                                      
//      FUNCTION NAME : validateTPByORName()                 
//                                                                      
//      MEMBER : X500Entry                                              
//                                                                      
//      AUTHOR : John Tebbutt                                           
//                                                                      
//      PURPOSE : To determine whether a trading partner, as specified by the
//      supplied ORName, is valid for an EDI user, as specified by a user
//      code and qualifier. The routine uses the X.500 search facility to
//      determine whether any TP entry exists with a matching ORName
//      attribute.
//                                                                      
//      DESCRIPTION : The DN of the EDI user is constructed, using the supplied
//      user code and qualifier, and the predefined pathname for the X.500
//      entry under which all EDI users' entries exist. This is used as the
//      base object of the search operation. The search filter is constructed,
//      indicating that matching entries must have object class MHS-EDI or
//      MHS-IPM, and ORName as specified. The entryInformationSelection
//      parameter to the search operation is constructed so as to require a
//      minimum of information. The search operation is invoked. If a match
//      is found, SUCCESS is returned, otherwise UNSUCCESSFUL.
//                                                                      
//      RETURNED VALUES :                                               
//      The function returns a STATUS code. The value SUCCESS indicates that
//      a match was found. Any other value indicates failure. 
//                                                                      
//      SIDE EFFECTS : On a bad day, it can make Paul seem like a nice
//      guy... 
//                                                                      
//      BUGS, WARNINGS AND SPECIAL COMMENTS :                           
//                                                                      
//      MEMORY MANAGEMENT :                                             
//                                                                      
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

STATUS X500Entry::validateTPByORName( char *uCode, char *uQual, 
													OM_public_object oRName)
{
        char                *eDIUserCN;   // common name of edi user entry
		DS_status			ds_status;	  // status returned from search op
        OM_workspace        workspace;    // workspace for objects            
        OM_private_object   session;      // session for directory operations 
        OM_private_object   result;       // result of search operation         
        OM_sint             invoke_id;    // Invoke-ID of the search operation  
		STATUS				rVal;		  // value returned from function
        char                *myname = 	"validateTPByORName";

		tellme( myname, "Hello, world\n" );

    //
    // Build the Distinguished Name of the base object of the search
	// operation.
    // NOTE: The format of the DN is as follows: c="US",o="NIST",ou="EDI",
    // EDIUser=uCode"-"uQual
    //

        //
        // Construct the purported common name of the EDI user entry
        //
        if( ( eDIUserCN = createName( uCode, uQual ) ) == '\0' ){

            tellme( myname, "Cannot create eDIUserCN\n" );
        }

		tellme( myname, "Country object\n" );
 
// country object
		static OM_descriptor country[4];

		country[0].type = OM_CLASS;
		country[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
		country[0].value.string.length = OMP_LENGTH( DS_C_AVA );
		country[0].value.string.elements = OMP_D_DS_C_AVA;

		country[1].type = DS_ATTRIBUTE_TYPE;
		country[1].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
		country[1].value.string.length = OMP_LENGTH( DS_A_COUNTRY_NAME );
		country[1].value.string.elements = OMP_D_DS_A_COUNTRY_NAME;

		country[2].type = DS_ATTRIBUTE_VALUES;
		country[2].syntax = OM_S_PRINTABLE_STRING;
		country[2].value.string.length = sizeof( "US" ) - 1;
		country[2].value.string.elements = "US";

		country[3].type = OM_NO_MORE_TYPES;
//		country[3].syntax = 0;
//		country[3].value.string.length = 0;
//		country[3].value.string.elements = 0;

        tellme( myname, "Org object\n" ); 

// organization object
        static OM_descriptor organization[4];

        organization[0].type = OM_CLASS;
        organization[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        organization[0].value.string.length = OMP_LENGTH( DS_C_AVA );
        organization[0].value.string.elements = OMP_D_DS_C_AVA;
 
        organization[1].type = DS_ATTRIBUTE_TYPE;
        organization[1].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        organization[1].value.string.length = OMP_LENGTH( DS_A_ORGANIZATION_NAME );
        organization[1].value.string.elements = OMP_D_DS_A_ORGANIZATION_NAME;
 
        organization[2].type = DS_ATTRIBUTE_VALUES;
        organization[2].syntax = OM_S_TELETEX_STRING;
        organization[2].value.string.length = sizeof( "NIST" ) - 1;
        organization[2].value.string.elements = "NIST";
 
        organization[3].type = OM_NO_MORE_TYPES;
//      organization[3].syntax = 0;
//      organization[3].value.string.length = 0;
//      organization[3].value.string.elements = 0;
 
        tellme( myname, "Org Unit object\n" ); 

// organizational_unit object
        static OM_descriptor organizational_unit[4];
 
        organizational_unit[0].type = OM_CLASS;
        organizational_unit[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        organizational_unit[0].value.string.length = OMP_LENGTH( DS_C_AVA );
        organizational_unit[0].value.string.elements = OMP_D_DS_C_AVA;
 
        organizational_unit[1].type = DS_ATTRIBUTE_TYPE;
        organizational_unit[1].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        organizational_unit[1].value.string.length = OMP_LENGTH( DS_A_ORGANIZATIONAL_UNIT_NAME );
        organizational_unit[1].value.string.elements = OMP_D_DS_A_ORGANIZATIONAL_UNIT_NAME;
 
        organizational_unit[2].type = DS_ATTRIBUTE_VALUES;
        organizational_unit[2].syntax = OM_S_TELETEX_STRING;
        organizational_unit[2].value.string.length = sizeof( "EDI" ) - 1;
        organizational_unit[2].value.string.elements = "EDI";
 
        organizational_unit[3].type = OM_NO_MORE_TYPES;
//      organizational_unit[3].syntax = 0;
//      organizational_unit[3].value.string.length = 0;
//      organizational_unit[3].value.string.elements = 0;

        tellme( myname, "eDIUserName object\n" ); 

// eDIUserName object
//
// Temporary constant name, DS_A_EDI_NAME, 3/29/93
//
// edi-name ATTRIBUTE
//      WITH ATTRIBUTE SYNTAX caseExactStringSyntax
//      SINGLE VALUE
//      ::= id-dat-edi-routing-address  -- ::= {id-dat 1}
//                                         ::= {id-dir 1 1}
//                                         ::= {id-edims 15 1 1}
//                                         ::= {joint-iso-ccitt mhs-motis(6)
//                                                  edims(7) 15 1 1}
//
        static OM_descriptor eDIUserName[4];
 
        eDIUserName[0].type = OM_CLASS;
        eDIUserName[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        eDIUserName[0].value.string.length = OMP_LENGTH( DS_C_AVA );
        eDIUserName[0].value.string.elements = OMP_D_DS_C_AVA;
 
        eDIUserName[1].type = DS_ATTRIBUTE_TYPE;
        eDIUserName[1].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        eDIUserName[1].value.string.length = OMP_LENGTH( DS_A_ORGANIZATIONAL_UNIT_NAME );
        eDIUserName[1].value.string.elements = OMP_D_DS_A_ORGANIZATIONAL_UNIT_NAME;
 
        eDIUserName[2].type = DS_ATTRIBUTE_VALUES;
        eDIUserName[2].syntax = OM_S_TELETEX_STRING;
        eDIUserName[2].value.string.length = sizeof( eDIUserCN ) - 1;
        eDIUserName[2].value.string.elements = eDIUserCN;
 
        eDIUserName[3].type = OM_NO_MORE_TYPES;
//      eDIUserName[3].syntax = 0;
//      eDIUserName[3].value.string.length = 0;
//      eDIUserName[3].value.string.elements = 0;
 
        tellme( myname, "rdn1 object\n" );  

// rdn1 object
        static OM_descriptor rdn1[3];

        rdn1[0].type = OM_CLASS;
        rdn1[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        rdn1[0].value.string.length = OMP_LENGTH( DS_C_DS_RDN );
        rdn1[0].value.string.elements = OMP_D_DS_C_DS_RDN;

        rdn1[1].type = DS_AVAS;
        rdn1[1].syntax = OM_S_OBJECT;
        rdn1[1].value.object.padding = 0;
        rdn1[1].value.object.object = (OM_object)country;
 
        rdn1[2].type = OM_NO_MORE_TYPES;
//      rdn1[2].syntax = 0;
//      rdn1[2].value.object.padding = 0;
//      rdn1[2].value.object.object = 0;

        tellme( myname, "rdn2 object\n" );  

// rdn2 object
        static OM_descriptor rdn2[3];

        rdn2[0].type = OM_CLASS;
        rdn2[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        rdn2[0].value.string.length = OMP_LENGTH( DS_C_DS_RDN );
        rdn2[0].value.string.elements = OMP_D_DS_C_DS_RDN;

        rdn2[1].type = DS_AVAS;
        rdn2[1].syntax = OM_S_OBJECT;
        rdn2[1].value.object.padding = 0;
        rdn2[1].value.object.object = (OM_object)organization;

        rdn2[2].type = OM_NO_MORE_TYPES;
//      rdn2[2].syntax = 0;
//      rdn2[2].value.string.length = 0;
//      rdn2[2].value.string.elements = 0;

        tellme( myname, "rdn3 object\n" );  

// rdn3 object
        static OM_descriptor rdn3[3];

        rdn3[0].type = OM_CLASS;
        rdn3[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        rdn3[0].value.string.length = OMP_LENGTH( DS_C_DS_RDN );
        rdn3[0].value.string.elements = OMP_D_DS_C_DS_RDN;

        rdn3[1].type = DS_AVAS;
        rdn3[1].syntax = OM_S_OBJECT;
        rdn3[1].value.object.padding = 0;
        rdn3[1].value.object.object = (OM_object)organizational_unit;

        rdn3[2].type = OM_NO_MORE_TYPES;
//      rdn3[2].syntax = 0;
//      rdn3[2].value.string.length = 0;
//      rdn3[2].value.string.elements = 0;

        tellme( myname, "rdn4 object\n" );  

// rdn4 object
        static OM_descriptor rdn4[3];

        rdn4[0].type = OM_CLASS;
        rdn4[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        rdn4[0].value.string.length = OMP_LENGTH( DS_C_DS_RDN );
        rdn4[0].value.string.elements = OMP_D_DS_C_DS_RDN;
 
        rdn4[1].type = DS_AVAS;
        rdn4[1].syntax = OM_S_OBJECT;
        rdn4[1].value.object.padding = 0;
        rdn4[1].value.object.object = (OM_object)eDIUserName;

        rdn4[2].type = OM_NO_MORE_TYPES;
//      rdn4[2].syntax = 0;
//      rdn4[2].value.string.length = 0;
//      rdn4[2].value.string.elements = 0;

        tellme( myname, "name object\n" );  

// name object
        OM_descriptor name[6];

        name[0].type = OM_CLASS;
        name[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        name[0].value.string.length = OMP_LENGTH( DS_C_DS_DN );
        name[0].value.string.elements = OMP_D_DS_C_DS_DN;
 
        name[1].type = DS_RDNS;
        name[1].syntax = OM_S_OBJECT;
        name[1].value.object.padding = 0;
        name[1].value.object.object = (OM_object)rdn1;

        name[2].type = DS_RDNS;
        name[2].syntax = OM_S_OBJECT;
        name[2].value.object.padding = 0;
        name[2].value.object.object = (OM_object)rdn2;

        name[3].type = DS_RDNS;
        name[3].syntax = OM_S_OBJECT;
        name[3].value.object.padding = 0;
        name[3].value.object.object = (OM_object)rdn3;

        name[4].type = DS_RDNS;
        name[4].syntax = OM_S_OBJECT;
        name[4].value.object.padding = 0;
        name[4].value.object.object = (OM_object)rdn4;

        name[5].type = OM_NO_MORE_TYPES;
//      name[5].syntax = 0;
//      name[5].value.string.length = 0;
//      name[5].value.string.elements = 0;



    //
    // Public Object ("Descriptor List") for Entry-Information-Selection
    // argument to ds_search().
    //
    // We require no values of no attributes...
    //
// selection object
        tellme( myname, "selection object\n" );  


		OM_descriptor *selection = DS_SELECT_NO_ATTRIBUTES;

//      static OM_descriptor selection[4];

        //
        // Class attribute (This object represents entryInformation-
        // Selection information)
        //
//      selection[0].type = OM_CLASS;
//      selection[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
//      selection[0].value.string.length = 
//								OMP_LENGTH( DS_C_ENTRY_INFORMATION_SELECTION );
//      selection[0].value.string.elements = 
//										OMP_D_DS_C_ENTRY_INFORMATION_SELECTION;
 
        //
        // Attributes-Selected (No attributes)
        //
//		selection[1] = DS_SELECT_NO_ATTRIBUTES;

//        selection[1].type = DS_SELECT_NO_ATTRIBUTES;
//        selection[1].syntax = OM_S_BOOLEAN;
//        selection[1].value.boolean = OM_TRUE;

        //
        // Information-Type (Attribute types and values)
        //
//		selection[2].type = DS_INFORMATION_TYPE;                            
//        selection[2].syntax = OM_S_ENUMERATION;  
//        selection[2].value.enumeration = DS_TYPES_ONLY;

        //
        // Object terminator (That's all, folks!)
        //
//        selection[3].type = OM_NO_MORE_TYPES;

        tellme( myname, "filter item\n" );  

    //
    // Filter item public object for inclusion in the search filter
    //
	static OM_descriptor filter_item[5];

    //
    // Set the type and syntax of the filter item object
    //
	    filter_item[0].type = OM_CLASS;
	    filter_item[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
	    filter_item[0].value.string.length = OMP_LENGTH( DS_C_FILTER_ITEM );
	    filter_item[0].value.string.elements = OMP_O_DS_C_FILTER_ITEM;

    //
    // The filter item type
    //
	    filter_item[1].type = DS_FILTER_ITEM_TYPE; 
	    filter_item[1].syntax = OM_S_ENUMERATION; 
	    filter_item[1].value.enumeration = DS_EQUALITY; 

    //
    // The filter attribute type
    //
	    filter_item[2].type = DS_ATTRIBUTE_TYPE; 
	    filter_item[2].syntax = OM_S_OBJECT_IDENTIFIER_STRING; 
	    filter_item[2].value.string.length = 
									OMP_LENGTH( DS_A_OR_ADDRESSES ); 
	    filter_item[2].value.string.elements = OMP_O_DS_A_OR_ADDRESSES; 

    //
    // The filter attribute value
    //
	    filter_item[3].type = DS_ATTRIBUTE_VALUES; 
	    filter_item[3].syntax = OM_S_OBJECT; 
	    filter_item[3].value.object.padding = 0; 
	    filter_item[3].value.object.object = oRName; 

    //
    // Object terminator for filter item (That's all, folks!)
    //
		filter_item[4].type = OM_NO_MORE_TYPES;

    //
    // Public Object to represent the filter for the search operation.
    //
    // Note (6/29/93): For the time being and for the sake of simplicity,
    // the filter will specify only an equality match against the supplied
    // ORName. In reality, the search should also check that the object
    // class of the entry matches one of our MHS object classes.
    //

        tellme( myname, "filter object\n" );  

        static OM_descriptor filter[4];

        //
        // Set the type and syntax to reflect a search filter...
        //
		filter[0].type = OM_CLASS;
		filter[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
		filter[0].value.string.length = OMP_LENGTH( DS_C_FILTER );
		filter[0].value.string.elements = OMP_O_DS_C_FILTER;

		//
		// The filter item
		//
		filter[1].type = DS_FILTERS;
		filter[1].syntax = OM_S_OBJECT;
		filter[1].value.object.padding = 0;
		filter[1].value.object.object = filter_item;

        //
        // Filter type (ITEM)
        //
		filter[2].type = DS_FILTER_TYPE;
		filter[2].syntax = OM_S_ENUMERATION;
		filter[2].value.enumeration = DS_ITEM;

        //
        // Object terminator for filter (That's all, folks!)
        //
        filter[3].type = OM_NO_MORE_TYPES;


    //
    // Perform the Directory operations:
    // (1) Initialise the Directory Service and get an OM workspace.
    // (2) Bind a default directory session.
    // (3) Perform the search operation
    // (4) Terminate the directory session.
    //
    // Note: The ds_initialize() and ds_shutdown() calls were listed
    //       incorrectly in the XDS specification- ds_initialize() returns
    //       a workspace handle as its return value, not as an argument-
    //       and returns NULL is unsuccessful- hence the CHECK_DS_CALL()
    //       call was changed to report an error if workspace was equal to
    //       0 (NULL).  ds_shutdown() returns no values and takes no 
    //       values, hence making a CHECK_DS_CALL() didn't make much sense.
    //
    
    tellme( myname, "Calling ds_initialize\n" );  

    if( ( ( workspace = (OM_workspace)ds_initialize() ) == DS_NULL_RESULT ) ){

		cerr << myname << "Workspace initialization failure";
		return( UNSUCCESSFUL );
	}

    tellme( myname, "Calling ds_bind\n" );

    CHECK_DS_CALL( ds_bind( DS_DEFAULT_SESSION, &session ) );

    tellme( myname, "Calling ds_search\n" ); 

    ds_status = (DS_status)ds_search(  session, 
                        DS_DEFAULT_CONTEXT, 
                        name, 
                        DS_ONE_LEVEL,
                        filter,
                        OM_TRUE,
                        selection,
                        &result, 
                        &invoke_id );

	if( ds_status != DS_SUCCESS ){

		tellme( myname, "ds_search() error\n" );
	}

    tellme( myname, "Calling ds_unbind\n" ); 

    CHECK_DS_CALL( ds_unbind( session ) );

	//
	// Set the return value. If the base object was located (ds_search
	// returned DS_SUCCESS) and there are entries satisfying the search
	// criteria (one or more results are present), return SUCCESS (we have
	// no interest in the value of the results). Otherwise, if no results
	// are returned or the operation returns an error, we return UNSUCCESSFUL.
	// 
	rVal = UNSUCCESSFUL;

	if( ( ds_status == DS_SUCCESS ) && result ) rVal = SUCCESS;

    //
    //  We can now safely release all the private objects
    //  and the public objects we no longer need
    //
    CHECK_OM_CALL(om_delete(session));
    CHECK_OM_CALL(om_delete(result));

    tellme( myname, "Calling ds_shutdown\n" ); 

    ds_shutdown() ;

	return( rVal );

}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
//                                                                      
//      FUNCTION NAME : validateTPByLocalName()                 
//                                                                      
//      MEMBER : X500Entry                                              
//                                                                      
//      AUTHOR : John Tebbutt                                           
//                                                                      
//      PURPOSE : To determine whether a trading partner, as specified by the
//      supplied TP code and qualifier, is valid for an EDI user, as specified 
//      by a user code and qualifier. The routine uses the X.500 read facility 
//      to determine whether any TP entry exists with a matching distinguished
//      name.
//                                                                      
//      DESCRIPTION : The DN of the EDI user is constructed, using the supplied
//      user code and qualifier, and the predefined pathname for the X.500
//      entry under which all EDI users' entries exist. The DN of the TP
//      entry is constructed by simply appending the TP code and qualifier to
//      the EDI user DN.  The entryInformationSelection
//      parameter to the read operation is constructed so as to require a
//      minimum of information. The read operation is invoked. If the entry
//      exists, SUCCESS is returned, otherwise UNSUCCESSFUL.
//                                                                      
//      RETURNED VALUES :                                               
//      The function returns a STATUS code. The value SUCCESS indicates that
//      a match was found. Any other value indicates failure. 
//                                                                      
//      SIDE EFFECTS : On a bad day, it can make Paul seem like a nice
//      guy... 
//                                                                      
//      BUGS, WARNINGS AND SPECIAL COMMENTS :                           
//                                                                      
//      MEMORY MANAGEMENT :                                             
//                                                                      
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

STATUS X500Entry::validateTPByLocalName( char *uCode, char *uQual,
                                         char *tPCode, char *tPQual )
{
        char                *eDIUserCN;   // common name of edi user entry
        char                *eDITPCN;     // Common name of trading
                                          // partner entry 

        OM_workspace        workspace;    // workspace for objects            
        OM_private_object   session;      // session for directory operations 
        OM_private_object   result;       // result of read operation         
        OM_sint             invoke_id;    // Invoke-ID of the read operation  
		DS_status			ds_status;    // return value for x500 ops
        char                *myname = "validateTPByLocalName";

    //
    // Build the Distinguished Name.
    // NOTE: The format of the DN is as follows: c="US",o="NIST",ou="EDI",
    // EDIUser=uCode"-"uQual,cn=tPCode"-"tPQual.
    // By convention and for the time being, these DNs will identify aliases
    // to TP entries located under the node c="US",o="NIST",ou="EDI",ou="TPs".
    // This is to enable many users to reference multiple TPs without
    // duplicating TP information in the Directory.
    //

        //
        // Construct the purported common name of the EDI user entry
        //
        if( ( eDIUserCN = createName( uCode, uQual ) ) == '\0' ){

            tellme( myname, "Cannot create eDIUserCN\n" );
        }

        //
        // Construct the purported common name of the trading partner entry
        //
        if( ( eDITPCN = createName( tPCode, tPQual ) ) == '\0' ){

            tellme( myname, "Cannot create eDITPCN\n" );
        }

// country object

		tellme( myname, "Country object\n" );

		static OM_descriptor country[4];

		country[0].type = OM_CLASS;
		country[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
		country[0].value.string.length = OMP_LENGTH( DS_C_AVA );
		country[0].value.string.elements = OMP_D_DS_C_AVA;

		country[1].type = DS_ATTRIBUTE_TYPE;
		country[1].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
		country[1].value.string.length = OMP_LENGTH( DS_A_COUNTRY_NAME );
		country[1].value.string.elements = OMP_D_DS_A_COUNTRY_NAME;

		country[2].type = DS_ATTRIBUTE_VALUES;
		country[2].syntax = OM_S_PRINTABLE_STRING;
		country[2].value.string.length = sizeof( "US" ) - 1;
		country[2].value.string.elements = "US";

		country[3].type = OM_NO_MORE_TYPES;
//		country[3].syntax = 0;
//		country[3].value.string.length = 0;
//		country[3].value.string.elements = 0;

// organization object

		tellme( myname, "Org object\n" );

        static OM_descriptor organization[4];

        organization[0].type = OM_CLASS;
        organization[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        organization[0].value.string.length = OMP_LENGTH( DS_C_AVA );
        organization[0].value.string.elements = OMP_D_DS_C_AVA;
 
        organization[1].type = DS_ATTRIBUTE_TYPE;
        organization[1].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        organization[1].value.string.length = OMP_LENGTH( DS_A_ORGANIZATION_NAME );
        organization[1].value.string.elements = OMP_D_DS_A_ORGANIZATION_NAME;
 
        organization[2].type = DS_ATTRIBUTE_VALUES;
        organization[2].syntax = OM_S_TELETEX_STRING;
        organization[2].value.string.length = sizeof( "NIST" ) - 1;
        organization[2].value.string.elements = "NIST";
 
        organization[3].type = OM_NO_MORE_TYPES;
//      organization[3].syntax = 0;
//      organization[3].value.string.length = 0;
//      organization[3].value.string.elements = 0;
 
// organizational_unit object

		tellme( myname, "Org unit object\n" );

        static OM_descriptor organizational_unit[4];
 
        organizational_unit[0].type = OM_CLASS;
        organizational_unit[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        organizational_unit[0].value.string.length = OMP_LENGTH( DS_C_AVA );
        organizational_unit[0].value.string.elements = OMP_D_DS_C_AVA;
 
        organizational_unit[1].type = DS_ATTRIBUTE_TYPE;
        organizational_unit[1].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        organizational_unit[1].value.string.length = OMP_LENGTH( DS_A_ORGANIZATIONAL_UNIT_NAME );
        organizational_unit[1].value.string.elements = OMP_D_DS_A_ORGANIZATIONAL_UNIT_NAME;
 
        organizational_unit[2].type = DS_ATTRIBUTE_VALUES;
        organizational_unit[2].syntax = OM_S_TELETEX_STRING;
        organizational_unit[2].value.string.length = sizeof( "EDI" ) - 1;
        organizational_unit[2].value.string.elements = "EDI";
 
        organizational_unit[3].type = OM_NO_MORE_TYPES;
//      organizational_unit[3].syntax = 0;
//      organizational_unit[3].value.string.length = 0;
//      organizational_unit[3].value.string.elements = 0;

// eDIUserName object
//
// Temporary constant name, DS_A_EDI_NAME, 3/29/93
//
// edi-name ATTRIBUTE
//      WITH ATTRIBUTE SYNTAX caseExactStringSyntax
//      SINGLE VALUE
//      ::= id-dat-edi-routing-address  -- ::= {id-dat 1}
//                                         ::= {id-dir 1 1}
//                                         ::= {id-edims 15 1 1}
//                                         ::= {joint-iso-ccitt mhs-motis(6)
//                                                  edims(7) 15 1 1}
//

		tellme( myname, "EDI User Name object\n" );

        static OM_descriptor eDIUserName[4];
 
        eDIUserName[0].type = OM_CLASS;
        eDIUserName[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        eDIUserName[0].value.string.length = OMP_LENGTH( DS_C_AVA );
        eDIUserName[0].value.string.elements = OMP_D_DS_C_AVA;
 
        eDIUserName[1].type = DS_ATTRIBUTE_TYPE;
        eDIUserName[1].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        eDIUserName[1].value.string.length = OMP_LENGTH( DS_A_ORGANIZATIONAL_UNIT_NAME );
        eDIUserName[1].value.string.elements = OMP_D_DS_A_ORGANIZATIONAL_UNIT_NAME;
 
        eDIUserName[2].type = DS_ATTRIBUTE_VALUES;
        eDIUserName[2].syntax = OM_S_TELETEX_STRING;
        eDIUserName[2].value.string.length = sizeof( eDIUserCN ) - 1;
        eDIUserName[2].value.string.elements = eDIUserCN;
 
        eDIUserName[3].type = OM_NO_MORE_TYPES;
//      eDIUserName[3].syntax = 0;
//      eDIUserName[3].value.string.length = 0;
//      eDIUserName[3].value.string.elements = 0;
 
// tPName object

		tellme( myname, "TP Name object\n" );

        static OM_descriptor tPName[4];

        tPName[0].type = OM_CLASS;
        tPName[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        tPName[0].value.string.length = OMP_LENGTH( DS_C_AVA );
        tPName[0].value.string.elements = OMP_D_DS_C_AVA;

        tPName[1].type = DS_ATTRIBUTE_TYPE;
        tPName[1].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        tPName[1].value.string.length = OMP_LENGTH( DS_A_COMMON_NAME );
        tPName[1].value.string.elements = OMP_D_DS_A_COMMON_NAME;
 
        tPName[2].type = DS_ATTRIBUTE_VALUES;
        tPName[2].syntax = OM_S_TELETEX_STRING;
        tPName[2].value.string.length = sizeof( eDITPCN ) - 1;
        tPName[2].value.string.elements = eDITPCN;
 
        tPName[3].type = OM_NO_MORE_TYPES;
//      tPName[3].syntax = 0;
//      tPName[3].value.string.length = 0;
//      tPName[3].value.string.elements = 0;
 
// rdn1 object

		tellme( myname, "rdn1 object\n" );

        static OM_descriptor rdn1[3];

        rdn1[0].type = OM_CLASS;
        rdn1[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        rdn1[0].value.string.length = OMP_LENGTH( DS_C_DS_RDN );
        rdn1[0].value.string.elements = OMP_D_DS_C_DS_RDN;

        rdn1[1].type = DS_AVAS;
        rdn1[1].syntax = OM_S_OBJECT;
        rdn1[1].value.object.padding = 0;
        rdn1[1].value.object.object = (OM_object)country;
 
        rdn1[2].type = OM_NO_MORE_TYPES;
//      rdn1[2].syntax = 0;
//      rdn1[2].value.object.length = 0;
//      rdn1[2].value.object.object = 0;

// rdn2 object

		tellme( myname, "rdn2 object\n" );

        static OM_descriptor rdn2[3];

        rdn2[0].type = OM_CLASS;
        rdn2[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        rdn2[0].value.string.length = OMP_LENGTH( DS_C_DS_RDN );
        rdn2[0].value.string.elements = OMP_D_DS_C_DS_RDN;

        rdn2[1].type = DS_AVAS;
        rdn2[1].syntax = OM_S_OBJECT;
        rdn2[1].value.object.padding = 0;
        rdn2[1].value.object.object = (OM_object)organization;

        rdn2[2].type = OM_NO_MORE_TYPES;
//      rdn2[2].syntax = 0;
//      rdn2[2].value.string.length = 0;
//      rdn2[2].value.string.elements = 0;

// rdn3 object

		tellme( myname, "rdn3 object\n" );

        static OM_descriptor rdn3[3];

        rdn3[0].type = OM_CLASS;
        rdn3[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        rdn3[0].value.string.length = OMP_LENGTH( DS_C_DS_RDN );
        rdn3[0].value.string.elements = OMP_D_DS_C_DS_RDN;

        rdn3[1].type = DS_AVAS;
        rdn3[1].syntax = OM_S_OBJECT;
        rdn3[1].value.object.padding = 0;
        rdn3[1].value.object.object = (OM_object)organizational_unit;

        rdn3[2].type = OM_NO_MORE_TYPES;
//      rdn3[2].syntax = 0;
//      rdn3[2].value.string.length = 0;
//      rdn3[2].value.string.elements = 0;

// rdn4 object

		tellme( myname, "rdn4 object\n" );

        static OM_descriptor rdn4[3];

        rdn4[0].type = OM_CLASS;
        rdn4[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        rdn4[0].value.string.length = OMP_LENGTH( DS_C_DS_RDN );
        rdn4[0].value.string.elements = OMP_D_DS_C_DS_RDN;
 
        rdn4[1].type = DS_AVAS;
        rdn4[1].syntax = OM_S_OBJECT;
        rdn4[1].value.object.padding = 0;
        rdn4[1].value.object.object = (OM_object)eDIUserName;

        rdn4[2].type = OM_NO_MORE_TYPES;
//      rdn4[2].syntax = 0;
//      rdn4[2].value.string.length = 0;
//      rdn4[2].value.string.elements = 0;

// rdn5 object

		tellme( myname, "rdn5 object\n" );

        static OM_descriptor rdn5[3];

        rdn5[0].type = OM_CLASS;
        rdn5[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        rdn5[0].value.string.length = OMP_LENGTH( DS_C_DS_RDN );
        rdn5[0].value.string.elements = OMP_D_DS_C_DS_RDN;
 
        rdn5[1].type = DS_AVAS;
        rdn5[1].syntax = OM_S_OBJECT;
        rdn5[1].value.object.padding = 0;
        rdn5[1].value.object.object = (OM_object)tPName;

        rdn5[2].type = OM_NO_MORE_TYPES;
//      rdn5[2].syntax = 0;
//      rdn5[2].value.string.length = 0;
//      rdn5[2].value.string.elements = 0;

// name object

		tellme( myname, "name object\n" );

        OM_descriptor name[7];

        name[0].type = OM_CLASS;
        name[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
        name[0].value.string.length = OMP_LENGTH( DS_C_DS_DN );
        name[0].value.string.elements = OMP_D_DS_C_DS_DN;
 
        name[1].type = DS_RDNS;
        name[1].syntax = OM_S_OBJECT;
        name[1].value.object.padding = 0;
        name[1].value.object.object = (OM_object)rdn1;

        name[2].type = DS_RDNS;
        name[2].syntax = OM_S_OBJECT;
        name[2].value.object.padding = 0;
        name[2].value.object.object = (OM_object)rdn2;

        name[3].type = DS_RDNS;
        name[3].syntax = OM_S_OBJECT;
        name[3].value.object.padding = 0;
        name[3].value.object.object = (OM_object)rdn3;

        name[4].type = DS_RDNS;
        name[4].syntax = OM_S_OBJECT;
        name[4].value.object.padding = 0;
        name[4].value.object.object = (OM_object)rdn4;

        name[5].type = DS_RDNS;
        name[5].syntax = OM_S_OBJECT;
        name[5].value.object.padding = 0;
        name[5].value.object.object = (OM_object)rdn5;

        name[6].type = OM_NO_MORE_TYPES;
//      name[6].syntax = 0;
//      name[6].value.string.length = 0;
//      name[6].value.string.elements = 0;

    //
    // Public Object ("Descriptor List") for Entry-Information-Selection
    // argument to ds_read().
    //
    // We require no values of no attributes...
    //

// selection object

		tellme( myname, "selection object\n" );

        OM_descriptor *selection = DS_SELECT_NO_ATTRIBUTES;

        //
        // Class attribute (This object represents entryInformation-
        // Selection information)
        //
//      selection[0].type = OM_CLASS;
//      selection[0].syntax = OM_S_OBJECT_IDENTIFIER_STRING;
//      selection[0].value.string.length = 
//								OMP_LENGTH( DS_C_ENTRY_INFORMATION_SELECTION );
//      selection[0].value.string.elements = 
//										OMP_D_DS_C_ENTRY_INFORMATION_SELECTION;
 
        //
        // Attributes-Selected (All attributes)
        //
//      selection[1].type = DS_SELECT_NO_ATTRIBUTES;
//      selection[1].syntax = OM_S_BOOLEAN;
//      selection[1].value.boolean = OM_TRUE;

        //
        // Information-Type (Attribute types and values)
        //
//		selection[2].type = DS_INFORMATION_TYPE;                            
 //       selection[2].syntax = OM_S_ENUMERATION;  
  //      selection[2].value.enumeration = DS_TYPES_ONLY;

        //
        // Object terminator (That's all, folks!)
        //
   //     selection[3].type = OM_NO_MORE_TYPES;

    //
    // Perform the Directory operations:
    // (1) Initialise the Directory Service and get an OM workspace.
    // (2) Bind a default directory session.
    // (3) Issue read operation
    // (4) Terminate the directory session.
    //
    // Note: The ds_initialize() and ds_shutdown() calls were listed
    //       incorrectly in the XDS specification- ds_initialize() returns
    //       a workspace handle as its return value, not as an argument-
    //       and returns NULL is unsuccessful- hence the CHECK_DS_CALL()
    //       call was changed to report an error if workspace was equal to
    //       0 (NULL).  ds_shutdown() returns no values and takes no 
    //       values, hence making a CHECK_DS_CALL() didn't make much sense.
    //
     
    tellme( myname, "Calling ds_initialize\n" );

    if( ( ( workspace = (OM_workspace)ds_initialize() ) == DS_NULL_RESULT ) ){

		cerr << myname << "Workspace initialization failure";
		return( UNSUCCESSFUL );
	}

    tellme( myname, "Calling ds_bind\n" );

    CHECK_DS_CALL( ds_bind( DS_DEFAULT_SESSION, &session ) );

    tellme( myname, "Calling ds_read\n" );

    ds_status = (DS_status)ds_read( session, DS_DEFAULT_CONTEXT, name, 
									selection, &result, &invoke_id );

	if( ds_status != DS_SUCCESS ){

		tellme( myname, "ds_read() error\n" );
	}

    tellme( myname, "Calling ds_unbind\n" );

    CHECK_DS_CALL( ds_unbind( session ) );

    //
    //  We can now safely release all the private objects
    //  and the public objects we no longer need
    //
     
    CHECK_OM_CALL(om_delete(session)) ;
    CHECK_OM_CALL(om_delete(result)) ;

    tellme( myname, "Calling ds_shutdown\n" );

    ds_shutdown() ;

	//
	// If the operation was successful (ie an entry whose name corresponds
	// to the name supplied was found), return success. Otherwise clean up
	// the ds error returned from the operation (we do not need to examine its
	// contents) and return unsuccessful.
	//

	if( ds_status == DS_SUCCESS ){
	
    	tellme( myname, "Exiting SUCCESS\n" );

		return( SUCCESS );
	}
	else{
		
    	CHECK_OM_CALL( om_delete((OM_object) ds_status) );

    	tellme( myname, "Exiting UNSUCCESSFUL\n" );
		
		return( UNSUCCESSFUL );
	}

}   // End of validateTPByLocalName


//
// This is a dummy routine to satisfy the definition of validateTPByORName() 
// given in the class definition of X500Entry which accepts an ORName as
// a parameter
//
STATUS X500Entry::validateTPByORName( char *code, char *qual, ORName *name)
{
char	*myname = "validateTPByORName";

tellme( myname, form( "This is the dummy one: %s %s %d\n", code, qual, (int)name ) );

	return( SUCCESS );
}
