// 
// file: log.cc
//


#include <time.h>
#include <fstream.h>
#include <string.h>
#include <xom.h>
#include "../../../include/platform.h"
#include "../../../include/log.h"


//**********************************************************************
//
//  PROJECT:       EDI Communicaitons Platform
//  PROGRAMMER:    John J. Garguilo
//  CREATION DATE: 7/8/92
//  FILE NAME:    ~platform/src/lib/log/log.cc
//  DESCRIPTION:  This is the source file for the library class Log.
//                This file contains three methods named log_event(),
//                set_logfile(), set_loglevel().  
//                These methods will enable programs,
//                perhaps running concurrently, to write time-tagged 
//                text messages to a default or specified log file.   
//                The log file name can be changed at any time.
//
//***********************************************************************

//
// Method: set_logfile
// Class: Log
// Calls: new, close()
//
// Description:  This method allows the user to change the file used for
//               the logging of messages for a particular process.
//               The new log file will be name by the argument "logfile_name".
// 

STATUS Log::set_logfile(char *logfile_name)
{
   ofstream* newlogFile; // The new file to be opened.

   // Try to open the file identified by "logfile_name".  If the open
   // is not successful return UNSUCCESSFUL.

   newlogFile = new ofstream(logfile_name, ios::out|ios::app);
   // Check to make sure the open is successful.
   if (!newlogFile) { // open failed? 

#ifdef DEBUG
        cout << endl;
	cout << "\"" << logfile_name << "\" not successfully opened." << endl;
#endif
      return (UNSUCCESSFUL); // Keep the existing log file open.
      }

  // logfile_name was successfully opened so the previous log file
  // needs to be closed.  The new logfile will now be used for
  // all subsequent log_event calls.
  // However, make sure the logFile was previously opened before it 
  // can be closed.
  if (logfile)       
      logfile->close(); 

  // Give the new file stream to the "logfile" stream. 
   logfile = newlogFile; 
   log_event("set_logfile()", "Setting logfile name...",
             "New logfile named/created.");

#ifdef DEBUG
cout << "New logfile: \"" << logfile_name << "\" created." << endl;
#endif

  return(SUCCESS);
}

//
// Method: log_event
// Class: Log
// Calls: asctime(), strncpy(), strcpy(), strncat(), strcat(), time()
//
//  Description:  This function takes the argument information, formats it
//                into  a time - tagged line of text, and appends it to the
//                current log file.
// 

void Log::log_event (char *loc, char *msg, char *action)
{

  char date[DATE_LEN];    // The date and time when the logging took place. //
  char buffer[BUF_SIZE];  // Buffer to store the string to be written to    //
                          // the logfile.                                   //
  time_t totsecs;         // Number of total seconds since end of 1969.     //
  int i;                  // Counter.                                       //

  // First clear out the buffers. //
     for (i = 0; i < BUF_SIZE; i++)
       buffer[i] = ' ';
     for (i = 0; i < DATE_LEN; i++)
         date[i] = ' ';

  // Now record the date and time. //
     totsecs = time(NULL);

     strncpy(date, asctime(localtime(&totsecs)), (DATE_LEN - 1)); 
     date[DATE_LEN - 1] = '\0';

 // Copy the value of date into the buffer (which will eventually
 // be written). 
     strncpy(buffer, date, DATE_LEN); 

 // Concatenate the location which is logging the info to the buffer
 // which already contains the date.
  strncat(buffer, "\n\tLOC: ", 7);
  strncat(buffer, loc, strlen(msg));

 // Concatenate the message line of text on the end of the buffer 
 // which already contains the date.
  strncat(buffer, "\n\tMSG: ", 7);
  strncat(buffer, msg, strlen(msg));

 // Concatenate the "action" to be taken on the end of the buffer 
 // which already contains the date and message.
  strncat(buffer, "\n\tACTION: ", 10);
  strncat(buffer, action, strlen(action));

  // Now write the buffer to the appropriate log file. //
    (*logfile) << buffer << endl;
}


//
// Method: set_loglevel
// Class: Log
// Calls: none
//
//  Description:  This method sets the loglevel the user desires.
// 

void Log::set_loglevel(LOGLEVEL log_level) {

    loglevel = log_level;

}


void Log::log_event( char *loc, OM_return_code code, char *action ) {
	log_event( loc, ascii( code ), action );
}

//
// PRIVATE:  convert an OM_return_code to its ASCII equivalent.
//

char *Log::ascii( OM_return_code code ) {

	char	*ascii_code;

	switch ( code ) {

		case OM_ENCODING_INVALID:
			ascii_code = "ENCODING INVALID";
			break;
		case OM_FUNCTION_DECLINED:
			ascii_code = "FUNCTION DECLINED";
			break;
		case OM_FUNCTION_INTERRUPTED:
			ascii_code = "FUNCTION INTERRUPTED";
			break;
		case OM_MEMORY_INSUFFICIENT:
			ascii_code = "MEMORY INSUFFICIENT";
			break;
		case OM_NETWORK_ERROR:
			ascii_code = "NETWORK ERROR";
			break;
		case OM_NO_SUCH_CLASS:
			ascii_code = "NO SUCH CLASS";
			break;
		case OM_NO_SUCH_EXCLUSION:
			ascii_code = "NO SUCH EXCLUSION";
			break;
		case OM_NO_SUCH_MODIFICATION:
			ascii_code = "NO SUCH MODIFICATION";
			break;
		case OM_NO_SUCH_OBJECT:
			ascii_code = "NO SUCH OBJECT";
			break;
		case OM_NO_SUCH_RULES:
			ascii_code = "NO SUCH RULES";
			break;
		case OM_NO_SUCH_SYNTAX:
			ascii_code = "NO SUCH SYNTAX";
			break;
		case OM_NO_SUCH_TYPE:
			ascii_code = "NO SUCH TYPE";
			break;
		case OM_NO_SUCH_WORKSPACE:
			ascii_code = "NO SUCH WORKSPACE";
			break;
		case OM_NOT_AN_ENCODING:
			ascii_code = "NOT AN ENCODING";
			break;
		case OM_NOT_CONCRETE:
			ascii_code = "NOT CONCRETE";
			break;
		case OM_NOT_PRESENT:
			ascii_code = "NOT PRESENT";
			break;
		case OM_NOT_PRIVATE:
			ascii_code = "NOT PRIVATE";
			break;
		case OM_NOT_THE_SERVICES:
			ascii_code = "NOT THE SERVICES";
			break;
		case OM_PERMANENT_ERROR:
			ascii_code = "PERMANENT ERROR";
			break;
		case OM_POINTER_INVALID:
			ascii_code = "POINTER INVALID";
			break;
		case OM_SYSTEM_ERROR:
			ascii_code = "SYSTEM ERROR";
			break;
		case OM_TEMPORARY_ERROR:
			ascii_code = "TEMPORARY ERROR";
			break;
		case OM_TOO_MANY_VALUES:
			ascii_code = "TOO MANY VALUES";
			break;
		case OM_VALUES_NOT_ADJACENT:
			ascii_code = "VALUES NOT ADJACENT";
			break;
		case OM_WRONG_VALUE_LENGTH:
			ascii_code = "WRONG VALUE LENGTH";
			break;
		case OM_WRONG_VALUE_MAKEUP:
			ascii_code = "WRONG VALUE MAKEUP";
			break;
		case OM_WRONG_VALUE_NUMBER:
			ascii_code = "WRONG VALUE NUMBER";
			break;
//		case OM_WRONG_VALUE_POSITION:
//			ascii_code = "WRONG VALUE POSITION";
//			break;
		case OM_WRONG_VALUE_SYNTAX:
			ascii_code = "WRONG VALUE SYNTAX";
			break;
		case OM_WRONG_VALUE_TYPE:
			ascii_code = "WRONG VALUE TYPE";
			break;
		default:
			ascii_code = "UNRECOGNIZED OM_RETURN_CODE";
#ifdef	DEBUG
			cout << "Unrecognized om_return_code of " << code;
			cout << " received" << endl;
#endif
	}
	
	return ascii_code;
}
